C
C*****************************************************************
C
C     PROGRAM 3PARA (THREE PARAMETERS)
C     
C     THIS PROGRAM COMPUTES THE THREE PARAMETERS:
C
C        1) INITIAL STIFFNESS, Rki
C        2) ULTIMATE MOMENT, Mu
C        3) SHAPE PARAMETER, n
C
C     FOR FOLLOWING SEMI-RIGID CONNECTIONS:
C
C        1) TOP- AND SEAT-ANGLE CONNECTION
C        2) TOP- AND SEAT-ANGLE WITH WEB-ANGLE CONNECTION
C
C     PROGRAMMED BY KIM, SEUNG-EOCK
C                   STRUCTURES AREA, SCHOOL OF CIVIL ENGINEERING
C                   PURDUE UNIVERSITY
C
C     LAST CHANGED ON JUNE 1996
C
C****************************************************************
C
      IMPLICIT REAL (A-H,K-Z) 
      COMMON VOT,G2,TT,VOA,GY,TA
      EXTERNAL FUNC1,FUNC2
      OPEN (UNIT=5, FILE='CONN.DAT')
      OPEN (UNIT=6, FILE='CONN.OUT')
C
C     DATA INPUT AND WRITE
C
      DA=1.0
      TA=1.0
      KA=1.0
      GA=1.0
      READ (5,*) ITYPE,FY,E
      READ (5,*) LT,TT,KT,GT,W,BD 
      IF (ITYPE.EQ.2) READ (5,*) DA,TA,KA,GA
      LS=LT
      TS=TT
      KS=KT
      GS=GT
      WRITE (6,10)
      WRITE (6,*) ('          CONNECTION TYPE:')
      WRITE (6,*) ('          ----------------')
      IF (ITYPE.EQ.1) WRITE (6,20)
      IF (ITYPE.EQ.2) WRITE (6,30) 
      WRITE (6,*) ('          MATERIAL PROPERTY:')
      WRITE (6,*) ('          ------------------')
      WRITE (6,40) FY,E
      WRITE (6,*) ('          DIMENSIONS RELATED TO TOP ANGLE:')
      WRITE (6,*) ('          --------------------------------')
      WRITE (6,50) LT,TT,KT,GT,W,BD
      WRITE (6,*) ('          DIMENSIONS RELATED TO BOTTOM ANGLE:')
      WRITE (6,*) ('          -----------------------------------')
      WRITE (6,60) LS,TS,KS,GS,W
      IF (ITYPE.EQ.2) THEN 
	 WRITE (6,*) ('          DIMENSIONS RELATED TO WEB ANGLE:')
	 WRITE (6,*) ('          --------------------------------')
	 WRITE (6,70) DA,TA,KA,GA
      ENDIF
      WRITE (6,*) ('       OUTPUT OF THREE PARAMETERS')
      WRITE (6,*) ('       ==========================')
C
C     CALCULATION OF INITIAL STIFFNESS, Rki
C 
      LL=(BD-DA)/2.0
      EIT=E*LT*TT**3/12.0
      EIA=E*DA*TA**3/12.0       
      D1=BD+TS/2.0+TT/2.0
      D3=BD/2.0+TS/2.0
      G1=GT-W/2.0-TT/2.0
      G3=GA-W/2.0-TA/2.0
      RKIT=3.0*EIT*D1**2/(G1*(G1**2+0.78*TT**2))
      RKI=RKIT
      RKIA=6.0*EIA*D3**2/(G3*(G3**2+0.78*TA**2))
      IF(ITYPE.EQ.2) RKI=RKI+RKIA
C
C     CALCULATION OF ULTIMATE MOMENT, Mu
C       
      MOS=FY*LS*TS**2/4.0
      VOT=FY*LT*TT/2.0
      G2=GT-KT-W/2.0-TT/2.0
      VPTMIN=0.0
      VPTMAX=FY*LT*TT
      VPT=RTBIS(FUNC1,VPTMIN,VPTMAX)
      MPT=FY*LT*TT**2/4.0
      VOA=FY*TA/2.0
      GY=GA-KA
      VPYMIN=0.0
      VPYMAX=FY*DA*TA
      VPU=RTBIS(FUNC2,VPYMIN,VPYMAX)
      VPA=0.5*(VPU+VOA)*DA
      D2=BD+TS/2.0+KT
      D4=DA*(2*VPU+VOA)/(3.0*(VPU+VOA))+LL+TS/2.0
      MU=MOS+MPT+VPT*D2
      IF (ITYPE.EQ.2) MU=MU+2.0*VPA*D4
C
C     CALCULATION OF SHAPE PARAMETER, n
C
      THETA0=MU/RKI
      IF (ITYPE.EQ.1) N=MAX(2.003*LOG10(THETA0)+6.070,0.302)
      IF (ITYPE.EQ.2) N=MAX(1.398*LOG10(THETA0)+4.631,0.827)
C
C     WRITE THREE PARAMETERS
C
      WRITE (6,80) RKI
      WRITE (6,90) MU
      WRITE (6,100) N
10    FORMAT (///,'       INPUT DATA',/,
     &            '       ==========',/)
20    FORMAT (/,14X,'TOP- AND SEAT- ANGLE CONNECTION',//)
30    FORMAT (/,14X,'TOP- AND SEAT- ANGLE WITH WEB-ANGLE ', 
     &        'CONNECTION',//)
40    FORMAT (/,14X,'Fy =',F8.4,4X,'E  =',F8.1,//,)
50    FORMAT (/,14X,'lt =',F8.4,4X,'tt =',F8.4,4X,'kt =',F8.4,//,
     &        14X,'gt =',F8.4,4X,'W  =',F8.4,4X,'d  =',F8.4,///)
60    FORMAT (/,14X,'ls =',F8.4,4X,'ts =',F8.4,4X,'ks =',F8.4,//,
     &        14X,'gs =',F8.4,4X,'W  =',F8.4,///)
70    FORMAT (/,14X,'da =',F8.4,4X,'ta =',F8.4,4X,'ka =',F8.4,//,
     &        14X,'ga =',F8.4,///)
80    FORMAT (/,14X,'INITIAL STIFFNESS, Rki = ',F10.2,/)
90    FORMAT (/,14X,'ULTIMATE MOMENT,   Mu  = ',F10.2,/)
100   FORMAT (/,14X,'SHAPE PARAMETER,   n   = ',F10.3,/)
      STOP 
      END
C
C     FUNCTIONS
C
      FUNCTION RTBIS (FUNC,X1,X2)
      COMMON VOT,G2,TT,VOA,GY,TA
      PARAMETER (JMAX=10000)
      FMID=FUNC(X2)
      F=FUNC(X1)
      IF (F.LT.0.0) THEN
	RTBIS=X1
	DX=X2-X1
      ELSE
	RTBIS=X2
	DX=X1-X2
      ENDIF
      DO 110 J=1,JMAX
	DX=0.5*DX
	XMID=RTBIS+DX
	FMID=FUNC(XMID)
	IF(FMID.LE.0.0) RTBIS=XMID
	IF(ABS(DX).LT.0.00001.OR.FMID.EQ.0.0) RETURN
110   CONTINUE
      END
C
      FUNCTION FUNC1(VPT)
	COMMON VOT,G2,TT
	FUNC1=(VPT/VOT)**4+(G2/TT)*(VPT/VOT)-1
      RETURN
      END
C
      FUNCTION FUNC2(VPY)
	COMMON VOT,G2,TT,VOA,GY,TA
	FUNC2=(VPY/VOA)**4+(GY/TA)*(VPY/VOA)-1
      RETURN
      END
 
