      PROGRAM PAAP
C
C********************************************************************
C                                                                    
C   PRACTICAL ADVANCED ANALYSIS PROGRAM (PAAP) FOR PRAME DESIGN       
C                                                                    
C********************************************************************
C
C   1) INPUT FILES  : DATA0, DATA1, DATA2
C   2) OUTPUT FILES : P.OUT1, P.OUT2
C   3) ANALYSIS     : SECOND-ORDER INELASTIC ANALYSIS
C   4) OUTPUT       : INCREMENTAL DISPLACEMENTS AND FORCES AT NODES
C   5) SCOPE        : 2-D FRAME AND TRUSS
C   6) CONSIDERATIONS OF KEY EFFECTS OF STEEL BEHAVIOR
C        o SECOND-ORDER : STABILITY FUNCTION
C        o RESIDUAL STRESSES : CRC TANGENT MODULUS
C        o GRADUAL YIELDING WITH FLEXURE : PARABOLIC FUNCTION
C        o GEOMETRIC IMPERFECTION  : EXPLICIT IMPERFECTION MODELING
C                                  : EQUIVALENT NOTIONAL LOADS
C                                  : FURTHER REDUCED TANGENT MODULUS
C        o CONNECTION NONLINEARITY : KISHI AND CHEN POWER MODEL
C   7) DESIGN        : EQUIVALENT TO LRFD
C   8) SEPARATE MEMBER CAPACITY CHECK : NOT NECESSARY 
C   9) PROGRAMMED BY : KIM, SEUNG-EOCK
C                      STRUCTURES AREA, SCHOOL OF CIVIL ENGINEERING
C                      PURDUE UNIVERSITY
C  10) LAST CHANGED  : JUNE 1996
C  11) VERSION       : 1.0
C  12) BASIS OF THIS PROGRAM : "PHINGE" PROGRAMMED BY RICHARD LIEW
C
C********************************************************************
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C    M A I N    P R O G R A M    C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
        IMPLICIT DOUBLE PRECISION (A-H,K,O-Z)
        COMMON /SOLVEB/ ISOLVE,IGEOIM,ILRFD
        COMMON /ETBLK/  IET
        COMMON /PHIBLK/ IPHI  
        OPEN (UNIT=11,FILE='DATA0',FORM='FORMATTED')
        OPEN (UNIT=4,FILE='P.OUT1',FORM='FORMATTED')
        OPEN (UNIT=5,FILE='P.OUT2',FORM='FORMATTED')
        READ (11,100) ISOLVE,IGEOIM,ILRFD
        READ (11,105) IET,IPHI
        IF (IGEOIM.EQ.0) WRITE (4,290) 
        IF (IGEOIM.EQ.1) WRITE (4,291) 
        IF (IGEOIM.EQ.2) WRITE (4,292) 
        IF (IGEOIM.EQ.3) WRITE (4,293) 
        IF (ILRFD.EQ.0) WRITE (4,390)
        IF (ILRFD.EQ.1) WRITE (4,391)
        WRITE (4,300)           
        WRITE (4,310)
        WRITE (4,330)
  
        CALL SSTAR
C
C    FORMAT STATEMENTS
C
100     FORMAT (3I5)
105     FORMAT (2I5)
290     FORMAT (' ','***********************************************',/
     &          ,T2,'*         NO GEOMETRIC IMPERFECTION           *',/
     &          ,T2,'***********************************************',/)  
291     FORMAT (' ','***********************************************',/
     &          ,T2,'*      EXPLICIT IMPERFECTION MODELING         *',/
     &          ,T2,'***********************************************',/)
292     FORMAT (' ','***********************************************',/
     &          ,T2,'*         EQUIVALENT NOTIONAL LOAD            *',/
     &          ,T2,'***********************************************',/)
293     FORMAT (' ','***********************************************',/
     &          ,T2,'*      FURTHER REDUCED TANGENT MODULUS        *',/
     &          ,T2,'***********************************************',/)
390     FORMAT (' ','***********************************************',/
     &          ,T2,'*       ANALYSIS (NO STRENGTH REDUCED)        *',/
     &          ,T2,'***********************************************',/)  
391     FORMAT (' ','***********************************************',/
     &          ,T2,'*         DESIGN (STRENGTH REDUCED)           *',/
     &          ,T2,'***********************************************',/)
300     FORMAT (' ','***********************************************',/
     &          ,T2,'* SECOND-ORDER REFINED PLASTIC HINGE ANALYSIS *',/
     &          ,T2,'***********************************************',/)
310     FORMAT (' ','=================================',/
     &          ,T2,'= CRC Tangent Modulus model     =')
330     FORMAT (' ','= Parabolic stiffness reduction =',/
     &          ,T2,'=================================',/)
C
1000    STOP
        END 
C
C
        SUBROUTINE SSTAR
C
C    THIS SUBROUTINE APPLYS THE REFINED PLASTIC HINGE METHOD FOR
C    FRAME ANALYSIS
        PARAMETER (MAXDOF=300,MAXTOF=300,MAXROF=100,MAXNODE=100,
     &             MAXTRS=150,MAXCNT=150)
        IMPLICIT DOUBLE PRECISION (A-H,K,O-Z)
        DOUBLE PRECISION INERT,MU,LOF,LOT,JMPC,JJMPC,MOM,MOMTMP,
     &                   NPOW
        DIMENSION KSTRCT(MAXDOF,MAXDOF),DGS(MAXDOF),RGS(MAXDOF),
     &            KFRAME(6,6),DGFTMP(6),
     &            KTRUSS(4,4),RGSM(MAXNODE,3),DGSM(MAXNODE,3),
     &            DGF(6),NCODE(6),IDOF(MAXNODE,3),
     &            MU(MAXCNT),RKI(MAXCNT),NPOW(MAXCNT),
     &            NORDER(MAXDOF),
     &            TCGF(3,6),TCGT(1,4),RGF(6),RGT(4),
     &            RGSTMP(MAXDOF),RGSIT1(MAXDOF),RGSIT2(MAXDOF),
     &            RGFPSD(6),RGSPSD(MAXDOF),
     &            DGSI(MAXDOF),DGSTMP(MAXDOF),
     &            ITRAN(MAXTOF),IROT(MAXROF)
        DIMENSION FORCE(MAXDOF),RGSI1(MAXDOF),RGSI2(MAXDOF),
     &            RGSF(MAXDOF),JHINGE(MAXROF),
     &            JJHING(MAXROF),JEND(MAXTOF),JMEM(MAXROF),
     &            JJEND(MAXTOF),JJMEM(MAXROF),
     &            JMPC(MAXROF),JJMPC(MAXROF),ISAME(MAXDOF),
     &            JFAIL(MAXTRS),JJFAIL(MAXTRS),
     &            ILD(MAXROF),ILDTMP(MAXROF),
     &            IMU(MAXROF),IMUTMP(MAXROF),IMUEND(MAXROF),
     &            THE(MAXROF),THETMP(MAXROF),
     &            MOM(MAXROF),MOMTMP(MAXROF),IPINC(MAXROF)
        DIMENSION RCF(MAXDOF,3),RCFTMP(MAXDOF,3),KC(MAXDOF,3,3),
     &            DELMPC(MAXDOF,2),RCFPSD(MAXDOF,3),JFLAGH(MAXDOF),
     &            RRCF(3)
        DIMENSION DELMTM(MAXDOF,2),JFLATM(MAXDOF)
        DIMENSION RCT(MAXTRS),RCTTMP(MAXTRS),DGT(4),
     &            DGTTMP(4)
        DIMENSION PHIAA(MAXDOF),PHIBB(MAXDOF),ETFRM(MAXDOF),
     &            ETT(MAXTRS), STORE(MAXDOF)
        COMMON /CNTB/ MU,NPOW,RKI
        COMMON /FRAMEB/ ZP,FYF,RGS,NDOFS,IDFRM
        COMMON /TRUSSB/ FYT,IDTRS,JFAIL,JJFAIL
        COMMON /INETRS/ ETT
        COMMON /STOPB/ ISTOP
        COMMON /POWERB/ MPOWER,NPOWER
        COMMON /HINGEB/ NCODE,JHINGE,JJHING,JJH,JJJH,ISAME
        COMMON /IDFRMB/ JMEM,JEND,JJMEM,JJEND
        COMMON /SJMPB/   JMPC,JJMPC
        COMMON /SCALEB/ RATIO,ISCALE,NSTEP
        COMMON /PPYB/ IPPY,IREDUE
        COMMON /RGFPSB/ RGFPSD
        COMMON /ILDB/ ILD,ILDTMP
        COMMON /THEMOB/ THE,THETMP,MOM,MOMTMP
        COMMON /THERB/ THETAR
        COMMON /FRMIN/ DELMPC,JFLAGH
        COMMON /FRMOUT/ RCFPSD,KC
        COMMON /INESTF/ PHIAA,PHIBB,ETFRM
        COMMON /SOLVEB/ ISOLVE,IGEOIM,ILRFD
        OPEN (UNIT=2,FILE='DATA1',FORM='FORMATTED')
        OPEN (UNIT=3,FILE='DATA2',FORM='FORMATTED')
        OPEN (UNIT=21, FILE='TEMP1')
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCC  TEMPORORY OUTPUT FOR PLOTTING                            CCC
C                                                                C
C     LOAD-DEFLECTION PLOT
C        OPEN (UNIT=5,FILE='OUT',FORM='FORMATTED')
C     FORCE POINT TRACE
c       OPEN (UNIT=9,FILE='OUT1',FORM='FORMATTED')
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C       INITIALIZATION
C
        CALL INITM  (KSTRCT,MAXDOF,MAXDOF)
        CALL INITM  (KFRAME,6,6)
        CALL INITM  (KTRUSS,4,4)
        CALL INITRV (DGS,MAXDOF)
        CALL INITRV (RGS,MAXDOF)
        CALL INITRV (DGF,6)
        CALL INITRV (DGFTMP,6)
        CALL INITRV (DGT,4)
        CALL INITRV (DGTTMP,4)
        CALL INITRV (RCT,MAXTRS)
        CALL INITRV (RCTTMP,MAXTRS)
        CALL INITIV (NCODE,6)
        CALL INITM  (RCF,MAXDOF,3)
        CALL INITM  (RCFTMP,MAXDOF,3)
        CALL INITM  (DELMPC,MAXDOF,2)
        CALL INITM  (DELMTM,MAXDOF,2)
        CALL INITM  (RCFPSD,MAXDOF,3)
        CALL INITIV (NORDER,MAXDOF)
        CALL INITM  (TCGF,3,6)
        CALL INITM  (TCGT,1,4)
        CALL INITRV (RGF,6)
        CALL INITRV (RGT,4)
        CALL INITRV (RGSTMP,MAXDOF)
        CALL INITRV (DGSTMP,MAXDOF)
        CALL INITRV (DGSI,MAXDOF)
        CALL INITIV (ITRAN,MAXTOF)
        CALL INITIV (IROT,MAXROF)
        CALL INITIV (JHINGE,MAXROF)
        CALL INITIV (JJHING,MAXROF)
        CALL INITIV (JMEM,MAXROF)
        CALL INITIV (JEND,MAXTOF)
        CALL INITRV (JMPC,MAXROF)
        CALL INITIV (JJMEM,MAXROF)
        CALL INITIV (JJEND,MAXTOF)
        CALL INITRV (JJMPC,MAXROF)
        CALL INITIV (JFAIL,MAXTRS)
        CALL INITIV (JJFAIL,MAXTRS)
        CALL INITRV (FORCE,MAXDOF)
        CALL INITRV (RGSI1,MAXDOF)
        CALL INITRV (RGSI2,MAXDOF)
        CALL INITRV (RGSIT1,MAXDOF)
        CALL INITRV (RGSIT2,MAXDOF)
        CALL INITRV (RGSF,MAXDOF)
CC*  PSEUDO FORCE DUE TO FORMATION OF PLASTIC HINGES *
        CALL INITRV (RGSPSD,MAXDOF)
        CALL INITRV (RGFPSD,6)
CC*  FLAG TO DETECT LOADING & UNLOADING OF CONNECTION *
CC*  TMP INDICATES PREVIOUS RECORD                    *
        CALL INITIV (ILD,MAXROF)
        CALL INITIV (ILDTMP,MAXROF)
        CALL INITIV (IMU,MAXROF)
        CALL INITIV (IMUTMP,MAXROF)
        CALL INITIV (IMUEND,MAXROF)
        CALL INITRV (THE,MAXROF)
        CALL INITRV (THETMP,MAXROF)
        CALL INITRV (MOM,MAXROF)
        CALL INITRV (MOMTMP,MAXROF)
        CALL INITIV (IPINC,MAXROF)
C*  ISAME GIVES THE POSSIBLE PLASTIC HINGES LOCATIONS *
C*  ISAME=2 WHEN CONNECTING NODE IS A RDOF
C*  ISAME=10 WHEN MOMENT IS APPLIED AT THE RDOF
        DO 111 I=1,MAXDOF
        JFLAGH(I)=4
        PHIAA(I)=1.0D0
        PHIBB(I)=1.0D0
111     ISAME(I)=1
C
C    READ IN THE NUMBER OF DEGREES OF FREEDOM OF THE STRUCTURE, NDOFS,
C    THE NUMBER OF LOAD INCREMENTS, NINCRE, THE NUMBER OF LOAD
C    SEQUENCES, NSEQN, AND FLAG FOR PRINTING ELEMENT FORCES AT EACH
C    LOAD STEP, IPRTFC
C
        READ (2,100) NDOFS,NINCRE,NSEQN,IPRTFC
C
C    READ IN THE NUMBER OF EACH TYPE OF ELEMENT
C
        READ (2,101) NCTYPE,NFTYPE,NTTYPE
        READ (2,101) NUMCNT,NUMFRM,NUMTRS
C
C    PRINT INPUT DATA
C
        WRITE (4,1001) NDOFS,NINCRE,NSEQN
        WRITE (4,1002) NCTYPE,NFTYPE,NTTYPE,
     &                 NUMCNT,NUMFRM,NUMTRS
C
        READ(21,3000) NNODE
3000    FORMAT(I5)
        CALL INITM  (DGSM,NNODE,3)
        CALL INITM  (RGSM,NNODE,3)
        DO 3002 NODE=1,NNODE
          READ(21,3004) IDOF(NODE,1),IDOF(NODE,2),IDOF(NODE,3)
3004      FORMAT(5X,3I5)
3002    CONTINUE
C
C    READ AND PRINT CONNECTION DATA
C
        IF (NCTYPE.EQ.0) GO TO 305
        WRITE (4,1003)

        DO 10 I=1,NCTYPE
          READ (2,102) IDCNT,MU(IDCNT),RKI(IDCNT),NPOW(IDCNT) 
          IF (ILRFD.EQ.1) MU(IDCNT)=MU(IDCNT)*0.90D0
          WRITE (4,1005) IDCNT,MU(IDCNT),RKI(IDCNT),NPOW(IDCNT)
10      CONTINUE

305     CONTINUE
C
C    READ IN THE INCREMENTAL LOAD VECTOR
C
        READ (2,135) (RGSI1(II),II=1,NDOFS)
C    IF NUMBER OF LOAD SEQUENCES EQUALS TWO, READ IN THE LOAD VECTOR
C    AT WHICH LOAD SEQUENCE 1 ENDS AND THE INCREMENTAL LOAD VECTOR
C    FOR LOAD SEQUENCE 2

        IF (NSEQN.EQ.2) THEN
          READ (2,135) (RGSF(II),II=1,NDOFS)
          READ (2,135) (RGSI2(II),II=1,NDOFS)
        ENDIF

C
C    INITIALIZATION
C
        NSTEP=0
        JJT=0
        JJR=0
        JJH=0
        JJJH=0
        LSEQN=1
        ISTOP=0
        MPOWER=0
        NPOWER=0
        RATIO=0.0D0
        THETAR=0.0D0
CC   LSCALE =1 IMPLIES TRUSS FAILURE OR CONNECTION UNLOADING OCCURS 
CC   LOAD IS SCALED BY A FACTOR OF 0.2
        LSCALE=0
        NSCALE=0
C

        DO 70 I=1,NDOFS
        RGSIT1(I)=RGSI1(I)
        IF (NSEQN.EQ.2) RGSIT2(I)=RGSI2(I)
70      CONTINUE

C
C    PROCEED TO NEXT LOAD STEP
C
        IREDUE=0
97      CONTINUE
C
C    SCALE DOWN THE LOAD INCREMENT IF PLASTIC HINGE FORM OR
C    UNLOADING OCCURS
C

        DO 73 II=1,NDOFS
        IF (LSEQN.EQ.1) THEN
        IF (JJH.GE.1.OR.LSCALE.EQ.1) RGSI1(II)=RGSI1(II)/5.0D0
        ELSE IF (LSEQN.EQ.2) THEN
        IF (JJH.GE.1.OR.LSCALE.EQ.1) RGSI2(II)=RGSI2(II)/5.0D0
        ENDIF
73      CONTINUE

        IF (NSCALE.EQ.1) THEN
        DO 74 II=1,NDOFS
        IF (LSEQN.EQ.1) THEN
        RGSI1(II)=RGSI1(II)/5.0D0
        RGSIT1(II)=RGSI1(II)
        ELSE IF (LSEQN.EQ.2) THEN
        RGSI2(II)=RGSI2(II)/5.0D0
        RGSIT2(II)=RGSI2(II)
        ENDIF
74      CONTINUE
        NSCALE=0
        ENDIF

C       INITIALIZATION
CC   ISCALE=1 IMPLIES CROSS SECTIONAL STRENGTH IS VIOLATED
CC   LOAD INCREMENTS ARE SCALED BY A FACTOR of 0.5
CC   JSCALE IS A SWITCH TO RESTORE THE PREVIOUS LOAD INCREMENT
        ISCALE=0
        IPPY=0
        JSCALE=0
        ITIME=0
96      CONTINUE
C    START ITERATION
C
        REWIND (3)
        CALL INITM (KSTRCT,MAXDOF,MAXDOF)
        CALL INITRV (RGSPSD,MAXDOF)
        CALL INITRV (FORCE,MAXDOF)
C
C    READ FRAME ELEMENT DATA,
C    FORMULATE AND ASSEMBLE THE FRAME ELEMENT STIFFNESS MATRICES
C
        IF (NUMFRM.EQ.0) GO TO 301
CC    READ AND PRINT FRAME ELEMENT DATA
        IF (NSTEP.EQ.0.AND.ITIME.EQ.0.AND.IREDUE.EQ.0)
     &  WRITE (4,1009)

        DO 1 I=1,NUMFRM
        READ (3,105) IDFRM,XO,YO,AREA,INERT,ZP,E,FYF,
     &               ICNT,IEND,IDCNT
        LOF=SQRT(XO**2+YO**2)
        IF (NSTEP.EQ.0.AND.ITIME.EQ.0.AND.IREDUE.EQ.0)
     &  WRITE (4,1011) IDFRM,XO,YO,LOF,AREA,INERT,ZP,E,FYF,
     &                 ICNT,IEND,IDCNT
        READ (3,110) (NCODE(II),II=1,6)
        IF (NSTEP.EQ.0.AND.ITIME.EQ.0.AND.IREDUE.EQ.0)
     &  THEN
          WRITE (4,1013) (NCODE(II),II=1,6)

CC    STORE TRANSLATIONAL DOF IN ITRAN AND STORE ROTATIONAL DOF IN IROT
           DO 23 J=1,6
            IF (NCODE(J).EQ.0) GO TO 23
            IF (MOD(J,3).NE.0) THEN
              DO 28 JJ=1,JJT
                IF (NCODE(J).EQ.ITRAN(JJ)) GO TO 23
28            CONTINUE
              JJT=JJT+1
              ITRAN(JJT)=NCODE(J)
            ELSE IF (MOD(J,3).EQ.0) THEN
              DO 29 JJ=1,JJR
              IF (NCODE(J).EQ.IROT(JJ)) THEN
                ISAME(NCODE(J))=ISAME(NCODE(J))+1
                IF (NSEQN.EQ.1.AND.ABS(RGSI1(NCODE(J))).GT.1.0D-10)
     &          ISAME(NCODE(J))=ISAME(NCODE(J))+10
                IF (NSEQN.EQ.2.AND.ABS(RGSI2(NCODE(J))).GT.1.0D-10)
     &          ISAME(NCODE(J))=ISAME(NCODE(J))+10
                GO TO 23
              ENDIF
29            CONTINUE
            JJR=JJR+1
            IROT(JJR)=NCODE(J)
            ENDIF
23        CONTINUE

        ENDIF

CC      GET ELEMENT DISPLACEMENTS, DGF FROM STRUCTURE DISPLACEMENTS, DGS
        DO 2 J=1,6
          IF (NCODE(J).EQ.0) THEN
            DGF(J)=0.0D0
          ELSE IF (NCODE(J).NE.0) THEN
            DGF(J)=DGS(NCODE(J))
          ENDIF
2       CONTINUE

        IF (ICNT.EQ.0) THEN
          RKT=1.0D+20
        ELSE IF (ICNT.EQ.1) THEN
          CALL RKTSTF (RKT,IDFRM,IDCNT,RCF(IDFRM,IEND))
          IF (ABS(RKT).LE.1.0D-05) THEN
            IMU(IDFRM)=1
            IMUEND(IDFRM)=IEND
            IF (RKI(IDCNT).LT.1.0D-05)
     &      IPINC(IDFRM)=1
          ENDIF
        ENDIF

        CALL FRAMEE (KFRAME,XO,YO,E,INERT,AREA,DGF,ICNT,RCF,
     &               IEND,RKT)
CC    ASSEMBLE THE STRUCTURE TANGENT STIFFNESS MATRIX
CC    AND ADD THE STRUCTURE INCREMENTAL PSEUDO FORCE VECTOR
        CALL ASSEMB (KSTRCT,KFRAME,NCODE,6,MAXDOF)
        CALL FMRGSP (RGSPSD,RGFPSD,NCODE,6,MAXDOF)
1       CONTINUE

301     CONTINUE
C
C    READ TRUSS ELEMENT DATA
C    FORMULATE AND ASSEMBLE THE TRUSS ELEMENT STIFFNESS MATRICES
C
        IF (NUMTRS.EQ.0) GO TO 302
CC    READ AND PRINT TRUSS ELEMENT DATA
        IF (NSTEP.EQ.0.AND.ITIME.EQ.0.AND.IREDUE.EQ.0)
     &  WRITE (4,1017)
        DO 4 I=1,NUMTRS
          READ (3,120) IDTRS,XO,YO,AREA,INERT,E,FYT,
     &                 (NCODE(II),II=1,4)
          LOT=SQRT(XO**2+YO**2)
        IF (NSTEP.EQ.0.AND.ITIME.EQ.0.AND.IREDUE.EQ.0)
     &  THEN
          WRITE (4,1019) IDTRS,XO,YO,LOT,AREA,INERT,E,FYT
          WRITE (4,1021) (NCODE(II),II=1,4)
CC    STORE TRANSLATIONAL DOF IN ITRAN
          DO 25 J=1,4
            IF (NCODE(J).EQ.0) GO TO 25
            DO 32 JJ=1,JJT
              IF (NCODE(J).EQ.ITRAN(JJ)) GO TO 25
32          CONTINUE
            JJT=JJT+1
            ITRAN(JJT)=NCODE(J)
25        CONTINUE
        ENDIF
CC    GET ELEMENT DISPLACEMENTS FROM STRUCTURE DISPLACEMENTS
        DO 5 J=1,4
          IF (NCODE(J).EQ.0) THEN
            DGT(J)=0.0D0
          ELSE IF (NCODE(J).NE.0) THEN
            DGT(J)=DGS(NCODE(J))
          ENDIF
5       CONTINUE
CC    CALCULATE ELEMENT BASIC FORCE AND FORM THE ELEMENT TANGENT
CC    STIFFNESS MATRIX
        CALL TRUSSE (KTRUSS,XO,YO,E,AREA,DGT,RCT)
CC    ASSEMBLE THE STRUCTURE TANGENT STIFFNESS MATRIX
        CALL ASSEMB (KSTRCT,KTRUSS,NCODE,4,MAXDOF)
4       CONTINUE
302     CONTINUE
C
C    PRINT NUMBERING FOR THE TRANSLATIONAL AND ROTATIONAL
C    DEGREES OF FREEDOM
C
        IF (NSTEP.EQ.0.AND.ITIME.EQ.0.AND.IREDUE.EQ.0)
     &  THEN
          WRITE (4,1035) JJT,(ITRAN(II),II=1,JJT)
          WRITE (4,1036) JJR,(IROT(II),II=1,JJR)
        ENDIF
C
C    DETERMINE THE INCREMENTAL FORCE VECTOR
C
99      DO 21 I=1,NDOFS
            IF (LSEQN.EQ.1) THEN
              FORCE(I)=RGSI1(I)-RGSPSD(I)
            ELSE IF (LSEQN.EQ.2) THEN
              FORCE(I)=RGSI2(I)-RGSPSD(I)
            ENDIF
21      CONTINUE
C
C    EVALUATE THE INCREMENTAL DISPLACEMENT VECTOR
C
        CALL SOLVDT (DGSI,DET,TRACE,KSTRCT,FORCE,NDOFS,MAXDOF,NORDER)
C        WRITE (*,*) "SOLVE FOR DISPLACEMENTS"
C
C    PRINT MESSAGE AND REDUCE LOAD INCREMENT IF DETERMINANT OF
C    THE TANGENT STIFFNESS MATRIX IS SINGULAR
C
        IF (DET.LT.0.0D0.OR.ISTOP.EQ.1) THEN
          WRITE (5,1040) NSTEP
          GO TO 996
        ENDIF
C
        GO TO 999
C
C
996     CONTINUE
C    IF REDUCTION OF LOAD INCREMENT IS NECESSARY----THEN
        IREDUE=IREDUE+1
C        WRITE (*,*) "Determinant is negative!"
CC    PRINT MESSAGE IF THE NUMBER OF LOAD REDUCTION IS GREATER THAN 3
        IF (IREDUE.GT.4) THEN
          WRITE (5,1039)
          GO TO 1000
        ENDIF
CC    SCALE DOWN THE INCREMENTAL LOAD VECTOR BY 1/5
        DO 43 I=1,NDOFS
          IF (LSEQN.EQ.1) THEN
            RGSI1(I)=RGSI1(I)/5.0D0
          ELSE IF (LSEQN.EQ.2) THEN
            RGSI2(I)=RGSI2(I)/5.0D0
          ENDIF
CC    RESTORE THE DISPLACEMENTS AND FORCES FROM PREVIOUS LOAD STEP
          DGS(I)=DGSTMP(I)
          RGS(I)=RGSTMP(I)
43      CONTINUE
        DO 50 I=1,JJH
          JHINGE(I)=JJHING(I)
          JMEM(I)=JJMEM(I)
          JEND(I)=JJEND(I)
          JMPC(I)=JJMPC(I)
50      CONTINUE
        JJH=JJJH
        DO 68 I=1,NUMTRS
          JFAIL(I)=JJFAIL(I)
          RCT(I)=RCTTMP(I)
68      CONTINUE
        DO 64 I=1,NUMFRM
          ILD(I)=ILDTMP(I)
          IMU(I)=IMUTMP(I)
          THE(I)=THETMP(I)
          MOM(I)=MOMTMP(I)
          RCF(I,1)=RCFTMP(I,1)
          RCF(I,2)=RCFTMP(I,2)
          RCF(I,3)=RCFTMP(I,3)
          DELMPC(I,1)=DELMTM(I,1)
          DELMPC(I,2)=DELMTM(I,2)
          JFLAGH(I)=JFLATM(I)
64      CONTINUE
        IMUEND(IDFRM)=0
        NSCALE=1
        NSTEP=NSTEP-1
        JSCALE=1
        GO TO 96
C
995     CONTINUE
C    IF SCALING OF LOAD INCREMENT IS NECESSARY----THEN
CC    SCALE DOWN THE LOAD INCREMENT
        WRITE (*,*) "LOAD INCREMENT IS SCALED"
CC    PRINT MESSAGE IF THE NUMBER OF LOAD SCALING IS GREATER THAN 100
        IF (ITIME.GT.50) GO TO 1000
CC
        DO 45 I=1,NDOFS
          IF (LSEQN.EQ.1) THEN
            RGSI1(I)=RATIO*RGSI1(I)
          ELSE IF (LSEQN.EQ.2) THEN
            RGSI2(I)=RATIO*RGSI2(I)
          ENDIF
CC    RESTORE THE VALUES FROM PREVIOUS LOAD STEP
          DGS(I)=DGSTMP(I)
          RGS(I)=RGSTMP(I)
45      CONTINUE
        DO 46 I=1,JJH
          JHINGE(I)=JJHING(I)
          JMEM(I)=JJMEM(I)
          JEND(I)=JJEND(I)
          JMPC(I)=JJMPC(I)
46      CONTINUE
        JJH=JJJH
        DO 67 I=1,NUMTRS
          JFAIL(I)=JJFAIL(I)
C         RCT(I)=RCTTMP(I)
67      CONTINUE
        DO 65 I=1,NUMFRM
          ILD(I)=ILDTMP(I)
          IMU(I)=IMUTMP(I)
          THE(I)=THETMP(I)
          MOM(I)=MOMTMP(I)
          RCF(I,1)=RCFTMP(I,1)
          RCF(I,2)=RCFTMP(I,2)
          RCF(I,3)=RCFTMP(I,3)
          DELMPC(I,1)=DELMTM(I,1)
          DELMPC(I,2)=DELMTM(I,2)
          JFLAGH(I)=JFLATM(I)
65      CONTINUE
        IMUEND(IDFRM)=0
        JSCALE=1
        ITIME=ITIME+1
        GO TO 96
C
999     CONTINUE
        IF(JSCALE.EQ.1.OR.NSTEP.EQ.0.) GO TO 51
C    PRINT INFORMATION FOR THE PRESENT LOAD STEP
        IF (NSTEP.EQ.1) WRITE (5,1037)
        WRITE (5,1032) NSTEP,LSEQN,DET,NPOWER,TRACE,MPOWER
        WRITE(5,3010)
C
C    PRINT GLOBAL DISPLACEMENTS AND FORCES
C
            I=0
         DO 3012 NODE=1,NNODE
         DO 3014 N=1,3
            IF(IDOF(NODE,N).EQ.0) THEN
               I=I+1
               RGSM(NODE,N)=RGS(I)
               DGSM(NODE,N)=DGS(I)
             ELSE
               RGSM(NODE,N)=0.0D0
               DGSM(NODE,N)=0.0D0
            ENDIF
3014     CONTINUE
3012     CONTINUE  
C
         DO 3020 NODE=1,NNODE
           WRITE(5,3022) NODE,RGSM(NODE,1),RGSM(NODE,2),RGSM(NODE,3)
           WRITE(5,3024) DGSM(NODE,1),DGSM(NODE,2),DGSM(NODE,3)        
3020     CONTINUE
C
C
        JJJH=JJH
        DO 47 I=1,JJJH
          IF (JMEM(I).EQ.JJMEM(I).AND.JEND(I).EQ.JJEND(I)) GO TO 47
CC   CHECK FOR MAXIMUM MOMENT IN THE CONNECTIONS
          DO 75 II=1,NUMFRM
          IF (IMU(II).EQ.1.AND.IMUTMP(II).EQ.0.AND.IPINC(II).NE.1) THEN
            WRITE (5,1059) II
            IMUTMP(II)=IMU(II)
            IF (II.EQ.JMEM(I).AND.IMUEND(II).EQ.JEND(I)) GO TO 47
          ENDIF
75        CONTINUE
CC   FORMATION OF PLASTIC HINGES AT A-TH OR B-TH END 
          IF (JEND(I).EQ.1) THEN
            WRITE (5,1051) JMEM(I)
          ELSE IF (JEND(I).EQ.2) THEN
            WRITE (5,1052) JMEM(I)
          ENDIF
47      CONTINUE
C
CC  TRUSS FAILS BY BUCKLING OR YIELDING
        DO 69 I=1,NUMTRS
          IF (JFAIL(I).EQ.1.AND.JJFAIL(I).EQ.0) THEN
            WRITE (5,1060) I
            LSCALE=1
          ENDIF
          IF (JFAIL(I).EQ.2.AND.JJFAIL(I).EQ.0) THEN
            WRITE (5,1061) I
            LSCALE=1
          ENDIF
69      CONTINUE
C
C
CC   UNLOADING OF CONNECTION
        DO 66 I=1,NUMFRM
          IF (ILD(I).EQ.1.AND.ILDTMP(I).EQ.0) THEN
            WRITE (4,1058) I
            LSCALE=1
          ENDIF
66      CONTINUE
C
C IPRTFC=1, PRINT ELEMENT FORCES   
C                                 
        IF (IPRTFC.EQ.0) GO TO 312
          WRITE (*,*) "LOAD STEP =",NSTEP
        REWIND (3)
        WRITE (5,1041)
CC    FRAME ELEMENT FORCES IN GLOBAL COORDINATE
        IF (NUMFRM.EQ.0) GO TO 311
CCC    PRINT HEADING
        WRITE (5,1042)
CCC    READ FRAME ELEMENT DATA
        DO 11 I=1,NUMFRM
          READ (3,105) IDFRM,XO,YO,AREA,INERT,ZP,E,FYF,
     &                 ICNT,IEND,IDCNT
          READ (3,110) (NCODE(II),II=1,6)
          DO 12 J=1,6
            IF (NCODE(J).EQ.0) THEN
CCC    GET ELEMENT DISPLACEMENTS FROM STRUCTURE DISPLACEMENTS
              DGF(J)=0.0D0
            ELSE IF (NCODE(J).NE.0) THEN
              DGF(J)=DGS(NCODE(J))
            ENDIF
12        CONTINUE
CCC    FORM TRANFORMATIN MATRIX AND TRANSFORM BASIC FORCES TO ELEMENT
CCC    FORCES IN GLOBAL COORDINATE, ASSEMBLE THE STRUCTURE FORCE
CCC    VECTOR IN GLOBAL COORDINATE
          DO 13 JJ=1,3
13        RRCF(JJ)=RCF(IDFRM,JJ)
          CALL FMTCGF (TCGF,XO,YO,DGF)
          CALL MUATV (RGF,TCGF,RRCF,3,6)
C
CC    PRINT ELEMENT FORCES IN GLOBAL COORDINATE
C
            WRITE (5,1043) IDFRM,(RGF(JJ),JJ=1,6)
C
CC    STORE INELASTIC REDUCTION FACTOR FOR PRINTING
          STORE(IDFRM)=ETFRM(IDFRM)/E
11      CONTINUE
C
311     CONTINUE
C
CC    TRUSS ELEMENT FORCES IN GLOBAL COORDINATE
        IF (NUMTRS.EQ.0) GO TO 312
CCC    PRINT HEADING
        WRITE (5,1045)
CCC    READ TRUSS ELEMENT DATA
        DO 14 I=1,NUMTRS
          READ (3,120) IDTRS,XO,YO,AREA,INERT,E,FYT,
     &                 (NCODE(II),II=1,4)
CCC    GET ELEMENT DISPLACEMENTS FROM STRUCTURE DISPLACEMENTS
        DO 15 J=1,4
          IF (NCODE(J).EQ.0) THEN
            DGT(J)=0.0D0
          ELSE IF (NCODE(J).NE.0) THEN
            DGT(J)=DGS(NCODE(J))
          ENDIF
15      CONTINUE
CCC    CALCULATE ELEMENT BASIC FORCE
CCC    FORM TRANSFORMATION MATRIX AND TRANSFORM THE ELEMENT BASIC
CCC    FORCE TO ELEMENT FORCES IN GLOBAL COORDINATE ASSEMBLE THE
CCC    STRUCTURE FORCE VECTOR IN GLOBAL COORDINATE
        CALL FMTCGT (TCGT,XO,YO,DGT)
        CALL MUATV (RGT,TCGT,RCT(IDTRS),1,4)
CC    CALCULATE TANGENT STIFFNESS REDUCTION FOR TRUSS ELEMENT
C       STORE1=ETT(IDTRS)/E
CC    PRINT ELEMENT FORCES IN GLOBAL COORDINATE
        WRITE (5,1046) IDTRS,(RGT(JJ),JJ=1,4)
c       WRITE (5,1046) IDTRS,(RGT(JJ),JJ=1,4),STORE1
14      CONTINUE
312     CONTINUE

C
C    UPDATE INFORMATION FOR NEXT ITERATION
        DO 98 I=1,NDOFS
          RGSTMP(I)=RGS(I)
          DGSTMP(I)=DGS(I)
98      CONTINUE
C
        DO 20 I=1,JJJH
          JJHING(I)=JHINGE(I)
          JJMEM(I)=JMEM(I)
          JJEND(I)=JEND(I)
          JJMPC(I)=JMPC(I)
20      CONTINUE
C
        DO 71 I=1,NUMTRS
          JJFAIL(I)=JFAIL(I)
          RCTTMP(I)=RCT(I)
71      CONTINUE
C
        DO 63 I=1,NUMFRM
          ILDTMP(I)=ILD(I)
          IMUTMP(I)=IMU(I)
          THETMP(I)=THE(I)
          MOMTMP(I)=MOM(I)
          RCFTMP(I,1)=RCF(I,1)
          RCFTMP(I,2)=RCF(I,2)
          RCFTMP(I,3)=RCF(I,3)
          DELMTM(I,1)=DELMPC(I,1)
          DELMTM(I,2)=DELMPC(I,2)
          JFLATM(I)=JFLAGH(I)
63      CONTINUE
51      CONTINUE
C
C    ADD DISPLACEMENT INCREMENTS TO THE CUMULATIVE DISPLACEMENTS
C
        DO 22 I=1,NDOFS
22      DGS(I)=DGS(I)+DGSI(I)
C 
        IF (ICONVG.EQ.1) THEN
          WRITE (5,1039)
          GO TO 1000
        ENDIF
C    CHECK FOR CONVERGENCE
        ICONVG=0
        ERRTM=0.0D0
        DO 38 I=1,JJT
        IF (ABS(DGS(ITRAN(I))).LT.1.0D-15) GO TO 38
        ERRT=ABS(DGSI(ITRAN(I))/DGS(ITRAN(I)))
        IF (ERRT.GT.ERRTM) ERRTM=ERRT
38      CONTINUE
        ERRRM=0.0D0
        DO 39 I=1,JJR
        IF (ABS(DGS(IROT(I))).LT.1.0D-15) GO TO 39
        ERRR=ABS(DGSI(IROT(I))/DGS(IROT(I)))
        IF (ERRR.GT.ERRRM) ERRRM=ERRR
39      CONTINUE
        IF (MAX(ERRTM,ERRRM).LT.0.5D-02) ICONVG=1
C
C     CALCULATE ELEMENT BASIC FORCES FOR NEXT ITERATION
C
        REWIND (3)
CC    FRAME ELEMENT FORCES IN GLOBAL COORDINATE
        IF (NUMFRM.EQ.0) GO TO 315
CCC    READ FRAME ELEMENT DATA
        DO 112 I=1,NUMFRM
          READ (3,105) IDFRM,XO,YO,AREA,INERT,ZP,E,FYF,
     &                 ICNT,IEND,IDCNT
          READ (3,110) (NCODE(II),II=1,6)
          DO 121 J=1,6
            IF (NCODE(J).EQ.0) THEN
CCC    GET ELEMENT DISPLACEMENTS FROM STRUCTURE DISPLACEMENTS
              DGF(J)=0.0D0
              DGFTMP(J)=0.0D0
            ELSE IF (NCODE(J).NE.0) THEN
              DGF(J)=DGS(NCODE(J))
              DGFTMP(J)=DGSTMP(NCODE(J))
            ENDIF
121       CONTINUE
CCC    CALCULATE ELEMENT BASIC FORCES
        CALL FRMBFC (RCF,DGF,DGFTMP,XO,YO,E,INERT,AREA,IDCNT,
     &               RCFTMP,IEND,IMU(IDFRM))
CCC    SCALE DOWN THE LOAD INCREMENT IF THE AXIAL FORCE IS LARGER THAN
CCC    THE YIELD LOAD OR THE MOMENT IS LARGER THAN THE PLASTIC MOMENT
        IF (IPPY.EQ.1) THEN
          IPPY=0
          RATIO=0.20D0
          GO TO 995
        ENDIF
        IF (ISCALE.EQ.1) THEN
          ISCALE=0
          GO TO 995
        ENDIF
C
C      SCALE DOWN THE LOAD INCREMENT IF THE CHANGE IN ET & PHI
C      EXCEEDS 0.1
C
          IF(ILRFD.EQ.1) THEN
          CPHIC=0.85D0
          CPHIB=0.90D0
          ELSE
          CPHIC=1.0D0
          CPHIB=1.0D0
          ENDIF
          PY=AREA*FYF*CPHIC
          XMP=ZP*FYF*CPHIB
          RCFA=RCF(IDFRM,3)
          PPY=ABS(RCFA)/PY
          CALL ETSTIF(RCFA,ET,E,PY)
          STORE2=ABS(ET-ETFRM(IDFRM))/E

          IF(STORE2.GT.0.1) THEN
          SCALE=0.10D0/STORE2
          RATIO = MAX(SCALE,0.50D0)
          WRITE(5,1221) IDFRM
          GO TO 995
          ENDIF

          IF (JEND(I).EQ.1) GO TO 991
          XM1=RCF(IDFRM,1)
          CALL ESTIFF (PPY,XM1,XMP,PHIA)
          STORE2=ABS(PHIA-PHIAA(IDFRM))

          IF(STORE2.GT.0.1) THEN
          SCALE=0.10D0/STORE2
          RATIO = MAX(SCALE,0.50D0)
          WRITE(5,1222) IDFRM
          GO TO 995 
          ENDIF
 
991       IF (JEND(I).EQ.2) GO TO 992
          XM2=RCF(IDFRM,2)
          CALL ESTIFF (PPY,XM2,XMP,PHIB)
          STORE3=ABS(PHIB-PHIBB(IDFRM))

          IF(STORE3.GT.0.1) THEN
          SCALE=0.10D0/STORE3
          RATIO = MAX(SCALE,0.50D0)
          WRITE(5,1223) IDFRM
          GO TO 995 
          ENDIF
 
992       CONTINUE
CCC    CHECK FOR CONNECTION UNLOADING
          IF ((NSTEP.NE.0).AND.
     &       (THE(IDFRM).LT.THETMP(IDFRM)).AND.
     &       ((((ABS(MOMTMP(IDFRM))-ABS(MOM(IDFRM)))/ABS(MOMTMP(IDFRM)))
     &       .GT.1.0D-02).OR.

     &       (MOM(IDFRM).LT.0.0D0.AND.MOMTMP(IDFRM).GE.0.0D0).OR.
     &       (MOM(IDFRM).GT.0.0D0.AND.MOMTMP(IDFRM).LE.0.0D0)))
     &       ILD(IDFRM)=1
112     CONTINUE
315     CONTINUE
C
CC    TRUSS ELEMENT FORCES IN GLOBAL COORDINATE
        IF (NUMTRS.EQ.0) GO TO 316
CCC    READ TRUSS ELEMENT DATA
        DO 141 I=1,NUMTRS
          READ (3,120) IDTRS,XO,YO,AREA,INERT,E,FYT,
     &                 (NCODE(II),II=1,4)
CCC    GET ELEMENT DISPLACEMENTS FROM STRUCTURE DISPLACEMENTS
        DO 151 J=1,4
          IF (NCODE(J).EQ.0) THEN
            DGT(J)=0.0D0
            DGTTMP(J)=0.0D0
          ELSE IF (NCODE(J).NE.0) THEN
            DGT(J)=DGS(NCODE(J))
            DGTTMP(J)=DGSTMP(NCODE(J))
          ENDIF
151     CONTINUE
CCC    CALCULATE ELEMENT BASIC FORCE
        CALL TRSBFC (RCT,DGT,DGTTMP,XO,YO,E,INERT,AREA,RCTTMP)
141     CONTINUE
316     CONTINUE
C
C    ADD INCREMENTAL FORCE VECTOR TO CUMULATIVE FORCE VECTOR
C
        ESMALL=0.0

        DO 52 I=1,NDOFS
        IF (LSEQN.EQ.1) THEN
        RGS(I)=RGSTMP(I)+RGSI1(I)

          IF (ABS(RGS(I)).GT.1D-06) THEN
          TEM=ABS(RGSI1(I)/RGS(I))
          IF(TEM.GT.ESMALL) ESMALL=TEM
          ENDIF

        ELSE IF (LSEQN.EQ.2) THEN
        RGS(I)=RGS(I)+RGSI2(I)

          IF (ABS(RGS(I)).GT.1D-06) THEN
          TEM=ABS(RGSI2(I)/RGS(I))
          IF(TEM.GT.ESMALL) ESMALL=TEM
          ENDIF

        ENDIF
52      CONTINUE
C
C    DETERMINE THE LOAD SEQUENCE
        IF (NSEQN.EQ.1) GO TO 94
        NN=0
        DO 49 I=1,NDOFS
        IF (ABS(RGS(I)-RGSF(I)).LE.ABS(1.0D-02*RGSF(I))) NN=NN+1
49      CONTINUE
        IF (NN.EQ.NDOFS) LSEQN=2
C
C    STOP THE EXECUTION IF THE NUMBER OF LOAD STEPS IS EQUAL TO THE
C    SPECIFIED VALUE
94      IF (NSTEP.EQ.NINCRE) GO TO 1000
C
C    RESTORE THE VALUE OF LOAD INCREMENT AFTER SCALING
          DO 113 I=1,NDOFS
            IF (LSEQN.EQ.1) THEN
              RGSI1(I)=RGSIT1(I)
            ELSE IF (LSEQN.EQ.2) THEN
              RGSI2(I)=RGSIT2(I)
            ENDIF
113       CONTINUE
        NSTEP=NSTEP+1
C
C    PROCEED TO THE NEXT LOAD STEP
        GO TO 97
C
1000    CONTINUE
C
C    PRINT THE SEQUENCE OF PLASTIC HINGE FORMATION
210   IF (JJH.GE.1) THEN
          WRITE (5,1053)
          DO 48 I=1,JJH
            IF (JEND(I).EQ.1) THEN
              WRITE (5,1054) I,JMEM(I)
            ELSE IF (JEND(I).EQ.2) THEN
              WRITE (5,1055) I,JMEM(I)
            ENDIF
48        CONTINUE
        ENDIF
C
C
C
        ENDFILE (4)
        CLOSE (UNIT=4,STATUS='KEEP')
C
C
C    FORMAT STATEMENTS
100     FORMAT (4I5)
101     FORMAT (5I5)
102     FORMAT (I5,3D12.0)
105     FORMAT (I5,7D12.0/3I5)
110     FORMAT (6I5)
120     FORMAT (I5,6D12.0/4I5)
135     FORMAT (8D12.0)
1001    FORMAT (' ','==================',/
     &          ,T2,'= STRUCTURE DATA =',/
     &          ,T2,'==================',/
     &        /,T6,'NUMBER OF DEGREES OF FREEDOM OF THE STRUCTURE =',I4,
     &        /,T6,'NUMBER OF LOAD INCREMENTS                     =',I4,
     &        /,T6,'NUMBER OF LOAD SEQUENCES                      =',I4)
1002    FORMAT (' ',
     &        /,T6,'NUMBER OF CONNECTION TYPES                    =',I4,
     &        /,T6,'NUMBER OF FRAME TYPES                         =',I4,
     &        /,T6,'NUMBER OF TRUSS TYPES                         =',I4,
     &       //,T6,'NUMBER OF CONNECTION ELEMENTS                 =',I4,
     &        /,T6,'NUMBER OF FRAME ELEMENTS                      =',I4,
     &        /,T6,'NUMBER OF TRUSS ELEMENTS                      =',I4,
     &        ///)
1003    FORMAT (' ',//,1X,'===================',/
     &                ,T2,'= CONNECTION DATA =',/
     &                ,T2,'===================',//)
1005    FORMAT (' ',T4,'----------------------',/
     &             ,T4,'- CONNECTION TYPE',I3,' -',/
     &             ,T4,'----------------------',/
     &,T8,'MU =',D12.5,6X,'RKI = ',D12.5,6X,'N = ',D12.5,//)
1009    FORMAT (' ',//,1X,'======================',/
     &                ,T2,'= FRAME ELEMENT DATA =',/
     &                ,T2,'======================',//)
1011    FORMAT (' ',T4,'---------------------',/
     &             ,T4,'- ELEMENT NUMBER',I3,' -',/
     &             ,T4,'---------------------',/
     &,T8,'XO    =',D12.5,4X,'YO   =',D12.5,4X,'LO   =',D12.5,/
     &,T8,'A     =',D12.5,4X,'I    =',D12.5,4X,'ZP   =',D12.5,/
     &,T8,'E     =',D12.5,4X,'FY   =',D12.5,/
     &,T8,'ICNT  =',I8,8X,'IEND =',I8,8X,'IDCNT=',I8,//)
1013    FORMAT (' ',T8,'ELEMENT   DOF :   1    2    3    4    5    6 ',/
     &,T8,'--------------------------------------------',/
     &,T8,'STRUCTURE DOF :',6(1X,I3,1X),///)
1017    FORMAT (' ',//,1X,'======================',/
     &                ,T2,'= TRUSS ELEMENT DATA =',/
     &                ,T2,'======================',//)
1019    FORMAT (' ',T4,'---------------------',/
     &             ,T4,'- ELEMENT NUMBER',I3,' -',/
     &             ,T4,'---------------------',/
     &          ,T8,'XO=',D12.5,T27,'YO=',D12.5,T46,'LO=',D12.5,/
     &          ,T8,'A =',D12.5,T27,'I =',D12.5,T46,'E =',D12.5,/
     &          ,T8,'FY=',D12.5,//)
1021    FORMAT (' ',T8,'ELEMENT   DOF :   1    2    3    4 ',/
     &          ,T8,'----------------------------------',/
     &          ,T8,'STRUCTURE DOF :',4(1X,I3,1X),///)
1032    FORMAT (//,T4,'-----------------',/
     &            ,T4,'- LOAD STEP',I4,' -',/
     &            ,T4,'-----------------',/
     &            ,T8,'LOAD SEQUENCE',I3,/
     &            ,T8,'DETERMINANT OF STIFFNESS MATRIX =',F6.3,
     &' TIMES 10 TO THE POWER ',I3,/
     &            ,T8,'TRACE OF STIFFNESS MATRIX       =',F6.3,
     &' TIMES 10 TO THE POWER ',I3)
3010    FORMAT (' ',/,11X,' NODE  ',3X,'ITEM',14X,'X',13X,'Y',13X,'R',/)
3022    FORMAT (' ',11X,I3,3X,'NODAL FORCE ',3X,D11.4,3X,D11.4,3X,D11.4)
3024    FORMAT (' ',17X,    'DISPLACEMENT',3X,D11.4,3X,D11.4,3X,D11.4,/)
1035    FORMAT (' ',//,1X,'================================',/
     &                ,T2,'= STRUCTURE DEGREES OF FREEDOM =',/
     &                ,T2,'================================',//
     &                ,T6,'NUMBER OF TRANSLATIONAL DOF =',I4,/
     &                ,T6,'DOF NUMBERING :',20I3,/,10(20X,20I3/))
1036    FORMAT (' ',T6,'NUMBER OF ROTATIONAL DOF    =',I4,/
     &             ,T6,'DOF NUMBERING :',20I3,/,10(20X,20I3/))
1037    FORMAT (' ',/////,1X,'============================',/
     &                   ,T2,'= LOAD-DEFLECTION BEHAVIOR =',/
     &                   ,T2,'============================')
1038    FORMAT ( //,2X,'>> SOLUTION DOES NOT CONVERGE IN 50 CYCLES AT LO
     &AD STEP <<',I4,'   ',/
     &             ,4X,'> WITH THE PRESENT LOAD INCREMENT, THEREFORE, L
     &OAD INCREMENT <<',/
     &             ,4X,'> IS REDUCED TO 1/5 OF PREVIOUS VALUE FOR THIS
     & LOAD STEP    ')
1039    FORMAT ( //,2X,'>> SOLUTION DOES NOT CONVERGE, A LIMIT STATE MAY
     & HAVE BEEN REACHED <<')
1040    FORMAT ( //,4X,'> DETERMINANT OF THE TANGENT STIFFNESS MATRIX IS
     & NEGATIVE AT LOAD',/
     &             ,4X,'> STEP',I4,' WITH THE PRESENT LOAD INCREMENT.'
     &'  THUS, LOAD INCREMENT',/  
     &             ,4X,'> IS REDUCED TO 1/5 OF PREVIOUS VALUE'
     &' FOR THIS LOAD STEP.')
1041    FORMAT (' ',/,T8,'ELEMENT FORCES IN GLOBAL COORDINATE',/
     &                ,T8,'-----------------------------------')
1042    FORMAT (' ',/,T10,'FRAME ELEMENT :',T40,'ELEMENT FORCES'/
     &               ,T10,'ELEMENT NUMBER',7X,'Fx',14X,'Fy',14X,'Mz',/)
1043    FORMAT (' ',T15,I3,3X,'i',T25,3(D14.7,2X),/,
     &              T15,6X, 'j',T25,3(D14.7,2X))
1045    FORMAT (' ',//,T10,'TRUSS ELEMENT :',15X,'ELEMENT FORCES',/,
     &              9X,'ELEMENT NO.',5X,'Fxi',10X,'Fyi',10X,'Fxj',10X,
     &              'Fyj',/)
1046    FORMAT (' ',13X,I3,4X,4(D10.3,3X))
1051    FORMAT ( //,4X,'* PLASTIC HINGE FORMS AT THE A-TH END OF FRAME
     &ELEMENT',I3,' *')
1052    FORMAT ( //,4X,'* PLASTIC HINGE FORMS AT THE B-TH END OF FRAME
     &ELEMENT',I3,' *')
1053    FORMAT (' ',//,1X,'=======================================',/
     &                ,T2,'= SEQUENCE OF PLASTIC HINGE FORMATION =',/
     &                ,T2,'=======================================',//
     &                ,T8,'PLASTIC HINGE',T36,'LOCATION',/)
1054    FORMAT (' ',T14,I2,T26,'A-TH END OF FRAME ELEMENT',I3)
1055    FORMAT (' ',T14,I2,T26,'B-TH END OF FRAME ELEMENT',I3)
1058    FORMAT ( //,1X,'* UNLOADING OCCURS AT THE CONNECTION WHICH IS C
     &ONNECTED TO FRAME ELEMENT',I3,' *')
1059    FORMAT ( //,4X,'* ULTIMATE MOMENT CAPACITY IS ATTAINED FOR THE
     &CONNECTION *',/
     &             ,4X,'* WHICH IS CONNECTED TO FRAME ELEMENT',I3,18X,
     &'*')
1060    FORMAT ( /,4X,'* TRUSS ELEMENT',I3,' HAS FAILED BY BUCKLING *
     &',/)
1061    FORMAT ( /,4X,'* TRUSS ELEMENT',I3,' HAS FAILED BY YIELDING *
     &',/)
1070    FORMAT (' ',/,1X,'INELASTIC STIFFNESS REDUCTION FACTORS FOR
     &FRAME ELEMENTS',/,
     &          3X,'FRAME NO.',4X,'PHIA',11X,'PHIB',12X,'ET')
1071    FORMAT (' ',4X,I3,3X,3(D12.5,4X))
1081    FORMAT (' ',2x,F12.5,2x,F12.5)
1082    FORMAT (' ',2x,F12.5,2x,F12.5,2x,F12.5)
1221    FORMAT ( /,4X,'* CHANGE IN ET EXCEEDS 0.1 AT FRAME
     & ELEMENT',I3,' *')
1222    FORMAT ( /,4X,'* CHANGE IN A-TH END STIFFNESS ETA EXCEEDS 0.1 AT FRAME
     & ELEMENT',I3,' *')
1223    FORMAT ( /,4X,'* CHANGE IN B-TH END STIFFNESS ETA EXCEEDS 0.1 AT FRAME
     & ELEMENT',I3,' *')
1224    FORMAT ( /,4X,'* PHIA=',F12.5,2x,'PHIAA',F12.5)
1225    FORMAT ( /,4X,'* PHIB=',F12.5,2x,'PHIBB',F12.5)
9990    FORMAT ( /,4X,6D12.5)
C
C
2000    RETURN
        END
C
C 
C
        SUBROUTINE FRMBFC (RCF,DGF,DGFTMP,XO,YO,E,INERT,AREA,IDCNT,
     &                     RCFTMP,IEND,IULTMO)
C    THIS SUBROUTINE IS CALLED BY SSTAR TO FIND THE BASIC FORCE VECTOR 
C    OF THE FRAME ELEMENT, RCF, FROM THE FRAME GLOBAL DISPLACEMENT 
C    VECTOR, DGF
C      RCF      ARRAY OF BASIC FORCES
C      DGF      ARRAY OF ELEMENT DISPLACEMENTS IN GLOBAL COORDINATE
C      DGFTMP   ARRAY OF ELEMENT DISPLACEMENTS IN GLOBAL COORDINATE OF
C               PREVIOUS LOAD STEP
C      RCFTMP   ARRAY OF ELEMENT FORCR IN LOCALL COORDINATE OF
C               PREVIOUS LOAD STEP
C      XO       HORIZONTAL PROJECTION OF THE UNDISPLACED ELEMENT
C      YO       VERTICAL PROJECTION OF THE UNDISPLACED ELEMENT
C      E        ELASTIC MODULUS
C      INERT    MOMENT OF INERTIA
C      AREA     CROSS SECTION AREA
C      IDCNT    CONNECTION IDENTIFIER
C      IEND     ELEMENT END IDENTIFIER
C      IULTMO   FLAG FOR THE ATTAINMENT OF CONNECTION ULTIMATE MOMENT
C
        PARAMETER (MAXDOF=300,MAXTOF=300,MAXROF=100)
        IMPLICIT DOUBLE PRECISION (A-H,K,O-Z)
        DOUBLE PRECISION LO,INERT,JMPC,JJMPC,MOM,MOMTMP,
     &                   MP,MPC,MMPC
        DIMENSION DCF(3),DCFTMP(3),DGF(6),DGFTMP(6),NCODE(6),
     &            RGS(MAXDOF),ISAME(MAXDOF),
     &            JHINGE(MAXROF),JJHING(MAXROF),
     &            JMEM(MAXROF),JJMEM(MAXROF),
     &            JEND(MAXTOF),JJEND(MAXTOF),
     &            JMPC(MAXROF),JJMPC(MAXROF),
     &            THE(MAXROF),THETMP(MAXROF),
     &            MOM(MAXROF),MOMTMP(MAXROF),
     &            ILD(MAXROF),ILDTMP(MAXROF),
     &            MMPC(2)
        DIMENSION DCFI(3),RCFI(3),RCFTMP(MAXDOF,3),TCGF(3,6),DGFI(6)
        DIMENSION KC(MAXDOF,3,3),RCF(MAXDOF,3),
     &            RCFPSD(MAXDOF,3),DELMPC(MAXDOF,2),JFLAGH(MAXDOF)
        COMMON /FRAMEB/ ZP,FYF,RGS,NDOFS,IDFRM
        COMMON /HINGEB/ NCODE,JHINGE,JJHING,JJH,JJJH,ISAME
        COMMON /IDFRMB/ JMEM,JEND,JJMEM,JJEND
        COMMON /SJMPB/   JMPC,JJMPC
        COMMON /SCALEB/ RATIO,ISCALE,NSTEP
        COMMON /PPYB/ IPPY,IREDUE
        COMMON /THEMOB/ THE,THETMP,MOM,MOMTMP
        COMMON /THERB/ THETAR
        COMMON /ILDB/ ILD,ILDTMP
        COMMON /FRMIN/ DELMPC,JFLAGH
        COMMON /FRMOUT/ RCFPSD,KC
        COMMON /SOLVEB/ ISOLVE,IGEOIM,ILRFD
C
C    JJH = Total number of hinges
C    JMEM = MEMBERS CONSIST OF PLASTIC HINGES FORMATION
C    JEND = 1 - CONNECTION AT A-TH END
C           2 - CONNECTION AT B-TH END
C    JHING = DOF AT WHICH PLASTIC HINGE OCCURS
C    N     = 0 - NO PLASTIC HINGE
C            1 - FORMATION OF ONE PLASTIC HINGE
C            2 - FORMATION OF TWO PLASTIC HINGE
C    NFLAGH= 3 - PLASTIC HINGE AT BOTH ENDS
C            1 - PLASTIC HINGE AT A-TH END
C            2 - PLASTIC HINGE AT B-TH END
C            0 - NO PLASTIC HINGE
C    IFLAGH= 1 - PLASTIC HINGE AT BOTH ENDS
C            2 - PLASTIC HINGE AT A-TH END
C            3 - PLASTIC HINGE AT B-TH END
C            4 - NO PLASTIC HINGE
C    IA    = PLASTIC HINGE IDENTIFIER AT A-TH END OF THE ELEMENT
C    IB    = PLASTIC HINGE IDENTIFIER AT B-TH END OF THE ELEMENT
C
C
C    INPUT : RCFTMP, RCFPSD
C            DGFTMP
C            DGF
C            KC
C    OUTPUT: RCF  DELMPC   JFLAGH
C
C    INITIALIZATION
C
        CALL INITRV (DCF,3)
C
C    DEFINE YIELD TOLERANCE 
C
        ERROR=5.0D-03
C

C    EVALUATE BASIC DISPLACEMENT VECTOR, DCF,
C    ASSUMING THERE IS NO PLASTIC HINGE IN THE MEMBER
        LO=SQRT(XO**2+YO**2)
        THETAO=ATAN2(YO,XO)
        XF=XO+DGF(4)-DGF(1)
        YF=YO+DGF(5)-DGF(2)
        THETA=ATAN2(YF,XF)
CC    CALCULATE PREVIOUS DATA
        XFP=XO+DGFTMP(4)-DGFTMP(1)
        YFP=YO+DGFTMP(5)-DGFTMP(2)
        THETAP=ATAN2(YFP,XFP)
        DCFTMP(1)=THETAO+DGFTMP(3)-THETAP
        DCFTMP(2)=THETAO+DGFTMP(6)-THETAP
        DCFTMP(3)=((2.0D0*XO+DGFTMP(4)-DGFTMP(1))*(DGFTMP(4)-
     &  DGFTMP(1))+(2.0D0*YO+DGFTMP(5)-DGFTMP(2))*(DGFTMP(5)
     &  -DGFTMP(2)))/(2.0D0*LO)
CC      CALCULATE FULL PLASTIC MOMENT CAPACITY AND SQUASH LOAD
          IF(ILRFD.EQ.1) THEN
          CPHIC=0.85D0
          CPHIB=0.90D0
          ELSE
          CPHIC=1.0D0
          CPHIB=1.0D0
          ENDIF
        MP=ZP*FYF*CPHIB
        PY=AREA*FYF*CPHIC
CC    EVALUATE TRANSFORMATION MATRIX, TCGF
        CALL FMTCGF(TCGF,XO,YO,DGFTMP)
CC    EVALUATE GLOBAL INCREMENTAL DISPLACEMENT VECTOR, DGFI
        DO 1 II=1,6
1       DGFI(II)=DGF(II)-DGFTMP(II)
CC    EVALUATE BASIC INCREMENTAL DISPLACEMENT VECTOR, DCFI
        CALL MULTMV (DCFI,TCGF,DGFI,3,6)
CC   EVALUATE BASIC INCREMENTAL FORCE VECTOR, RCFI 
        RCFI(1)=KC(IDFRM,1,1)*DCFI(1)+KC(IDFRM,1,2)*DCFI(2)
        RCFI(2)=KC(IDFRM,2,1)*DCFI(1)+KC(IDFRM,2,2)*DCFI(2)
        RCFI(3)=KC(IDFRM,3,3)*DCFI(3)
C   EVALUATE BASIC FORCE VECTOR, RCF
        DO 5 II =1,3
          RCF(IDFRM,II)=RCFI(II)+RCFTMP(IDFRM,II)
          DCF(II)=DCFI(II)+DCFTMP(II)
5       CONTINUE
C
C      ROTATE THE AXIAL FORCE COMPONENT TO C2 CONFIGURATION   CC
C 
        RCF(IDFRM,3)=RCF(IDFRM,3)-(RCF(IDFRM,1)+RCF(IDFRM,2))*
     &               SIN(THETA-THETAP)/LO
  
        RCFA=RCF(IDFRM,3)
C
C    IF A CONNECTION IS PRESENT
C
        IF (IEND.NE.0) THEN

        IF (ISOLVE.EQ.0) THEN
        CALL SFUNC (RCFA,S1,S2,LO,E,INERT)
        ET=E
        ELSE IF (ISOLVE.EQ.1) THEN
        CALL ETSTIF(RCFA,ET,E,PY)
        CALL SFUNC (RCFA,S1,S2,LO,ET,INERT)
        ENDIF

        CALL RKTSTF (RKT,IDFRM,IDCNT,RCFTMP(IDFRM,IEND))
        IF(RKT.LE.1.0D-05) RKT=1.0D-05
        THERI=ABS(RCFI(IEND)/RKT)

        IF (ABS(THERI).GT.0.0050D0) THEN
           ISCALE=1 
           WRITE (5,901)
           GO TO 1000
        ENDIF

        IF (IEND.EQ.1) THEN
          TMN=(ET*INERT/LO)*(S1*DCF(1)+S2*DCF(2))
          IF (TMN.GT.0.0D0) THEN
            DCFI(1)=DCFI(1)-THERI
          ELSE
            DCFI(1)=DCFI(1)+THERI
          ENDIF
        ENDIF

        IF (IEND.EQ.2) THEN
          TMN=(ET*INERT/LO)*(S2*DCF(1)+S1*DCF(2))
          IF (TMN.GT.0.0D0) THEN
            DCFI(2)=DCFI(2)-THERI
          ELSE
            DCFI(2)=DCFI(2)+THERI
          ENDIF
        ENDIF
        
        DCF(IEND)=DCFTMP(IEND)+DCFI(IEND)
        ENDIF

C
C    CHECK FOR THE PRESENCE OF PLASTIC HINGES
C
        N=0
        NFLAGH=0
        JA=0
        JB=0
        IA=0
        IB=0

        DO 10 I=1,JJH
        IF (IDFRM.EQ.JMEM(I).AND.JEND(I).EQ.1) THEN
        IA=I
        JA=1
        N=N+1
        NFLAGH=1
        IF (RCF(IDFRM,1).GT.0.0D0) THEN
        RCF(IDFRM,1)=JMPC(I)
        ELSE IF (RCF(IDFRM,1).LT.0.0D0) THEN
        RCF(IDFRM,1)=-JMPC(I)
        ENDIF
        GO TO 25
        ENDIF
10      CONTINUE

        IF (NCODE(3).EQ.0) GO TO 25

        DO 20 I=1,JJH
        IF (NCODE(3).EQ.JHINGE(I).AND.ISAME(NCODE(3)).EQ.2) THEN
        IA=I
        JA=2
        N=N+1
        NFLAGH=1
        IF (RCF(IDFRM,1).GT.0.0D0) THEN
        RCF(IDFRM,1)=JMPC(I)
        ELSE IF (RCF(IDFRM,1).LT.0.0D0) THEN
        RCF(IDFRM,1)=-JMPC(I)
        ENDIF
        GO TO 25
        ENDIF
20      CONTINUE

25      CONTINUE

        DO 30 I=1,JJH
        IF (IDFRM.EQ.JMEM(I).AND.JEND(I).EQ.2) THEN
        IB=I
        JB=1
        N=N+1
        NFLAGH=2
        IF (RCF(IDFRM,2).GT.0.0D0) THEN
        RCF(IDFRM,2)=JMPC(I)
        ELSE IF (RCF(IDFRM,2).LT.0.0D0) THEN
        RCF(IDFRM,2)=-JMPC(I)
        ENDIF
        GO TO 45
        ENDIF
30      CONTINUE

        IF (NCODE(6).EQ.0) GO TO 45

        DO 40 I=1,JJH
        IF (NCODE(6).EQ.JHINGE(I).AND.ISAME(NCODE(6)).EQ.2) THEN
        IB=I
        JB=2
        N=N+1
        NFLAGH=2
        IF (RCF(IDFRM,2).GT.0.0D0) THEN
        RCF(IDFRM,2)=JMPC(I)
        ELSE IF (RCF(IDFRM,2).LT.0.0D0) THEN
        RCF(IDFRM,2)=-JMPC(I)
        ENDIF
        GO TO 45
        ENDIF
40      CONTINUE

45      CONTINUE

C
C     EVALUATE THE PLASTIC MOMENT CAPACITY USING
C     AISC-LRFD CROSS-SECTION STRENGTH CURVE   
C 
        PPY=ABS(RCF(IDFRM,3))/PY
        XM1=ABS(RCF(IDFRM,1))/MP
        XM2=ABS(RCF(IDFRM,2))/MP
        EQN1=2.0D0*XM1/9.0D0
        EQN2=2.0D0*XM2/9.0D0

        IF(PPY.LE.EQN1) THEN
          F1=PPY/2.0D0+XM1
        ELSE IF (PPY.GT.EQN1) THEN
          F1=PPY+8.0D0*XM1/9.0D0
        ENDIF

        IF(PPY.LE.EQN2) THEN
          F2=PPY/2.0D0+XM2
        ELSE IF (PPY.GT.EQN2) THEN
          F2=PPY+8.0D0*XM2/9.0D0
        ENDIF

        IF (PPY.LE.0.2D0) THEN
          MPC=MP*(1.0D0-PPY/2.0D0)
        ELSE IF (PPY.GT.0.2D0.AND.PPY.LT.1.0D0) THEN
          MPC=MP*(1.0D0-PPY)*9.0D0/8.0D0
        ELSE IF (PPY.EQ.1.0D0) THEN
          WRITE (4,*) "IPPY = 1 FOR ELEMENT", IDFRM
          GO TO 1000
        ELSE IF (PPY.GT.1.0D0.AND.IREDUE.LE.9) THEN
          WRITE (5,98) IDFRM
          IPPY=1
          GO TO 1000
        ELSE IF (PPY.GT.1.0D0.AND.IREDUE.GT.9) THEN
          WRITE (5,99) IDFRM,(RGS(I),I=1,NDOFS)
          MPC=0.0D0
        ENDIF

C
C      IF ULTIMATE MOMENT CAPACITY OF CONNECTION IS LARGER THAN THAT
C      OF THE ADJOINING MEMBER
C
        IF(JA.EQ.1) F1=1.0D0
        IF(JB.EQ.1) F2=1.0D0

        IF (JA.EQ.2) THEN
          MMPC(1)=JMPC(IA)
          F1=1.0D0
        ELSE
          MMPC(1)=MPC
        ENDIF

        IF (JB.EQ.2) THEN
          MMPC(2)=JMPC(IB)
          F2=1.0D0
        ELSE
          MMPC(2)=MPC
        ENDIF

CC     REPLACE MPC BY RCF(IDFRM,IEND) IF THE ULTIMATE MOMENT CAPACITY OF THE
CC     CONNECTION IS LESS THAN THAT OF THE ADJOINING MEMBER
        IF (IEND.NE.0) THEN
          IF (IULTMO.EQ.1.AND.RCF(IDFRM,IEND).LT.MPC) THEN
            IF (IEND.EQ.1) THEN
              MMPC(1)=ABS(RCF(IDFRM,IEND))
c             MMPC(2)=MPC
            ELSE IF (IEND.EQ.2) THEN
c             MMPC(1)=MPC
              MMPC(2)=ABS(RCF(IDFRM,IEND))
            ENDIF
          ENDIF
        ENDIF
C
C     DETERMINE IF SCALING IS NECESSARY
C
        RATIO1=1.0D0
        RATIO2=1.0D0
        IF (MMPC(1).LT.1.0D-05) GO TO 36
        IF (F1.GT.(1.0D0+ERROR)) THEN
        PPYT=ABS(RCFTMP(IDFRM,3))/PY
        XM1T=ABS(RCFTMP(IDFRM,1))/MP
          EQN1=2.0D0*XM1T/9.0D0
          IF(PPYT.LE.EQN1) THEN
            F11=PPYT/2.0D0+XM1T
          ELSE IF (PPYT.GT.EQN1) THEN
            F11=PPYT+8.0D0*XM1T/9.0D0
          ENDIF
          RATIO1=1.0D0-(F1-1.0D0)/(F1-F11)
          DO 31 I=1,JJJH
            IF (IDFRM.EQ.JJMEM(I).AND.JJEND(I).EQ.1) RATIO1=1.0D0
            IF (NCODE(3).EQ.0) GO TO 31
            IF (NCODE(3).EQ.JJHING(I).AND.ISAME(NCODE(3)).EQ.2)
     &          RATIO1=1.0D0
31        CONTINUE
        ENDIF
        IF (MMPC(2).LT.1.0D-05) GO TO 36
        IF (F2.GT.(1.0D0+ERROR)) THEN
        PPYT=ABS(RCFTMP(IDFRM,3))/PY
        XM2T=ABS(RCFTMP(IDFRM,2))/MP
          EQN2=2.0D0*XM2T/9.0D0
          IF(PPYT.LE.EQN2) THEN
            F22=PPYT/2.0D0+XM2T
          ELSE IF (PPYT.GT.EQN2) THEN
            F22=PPYT+8.0D0*XM2T/9.0D0
          ENDIF
          RATIO2=1.0D0-(F2-1.0D0)/(F2-F22)
          DO 32 I=1,JJJH
          IF (IDFRM.EQ.JJMEM(I).AND.JJEND(I).EQ.2) RATIO2=1.0D0
          IF (NCODE(6).EQ.0) GO TO 32
          IF (NCODE(6).EQ.JJHING(I).AND.ISAME(NCODE(6)).EQ.2)
     &        RATIO2=1.0D0
32        CONTINUE
        ENDIF
        IF (ABS(RATIO1-RATIO2).LT.1.0D-05) GO TO 36
        RATIO=MIN(ABS(RATIO1),ABS(RATIO2))
        ISCALE=1
        IF (RATIO1.LT.RATIO2) THEN
          WRITE (5,96) IDFRM,RATIO
        ELSE IF (RATIO1.GT.RATIO2) THEN
          WRITE (5,97) IDFRM,RATIO
        ENDIF
        GO TO 1000
C
36      CONTINUE
C     CHECK FOR PLASTIC HINGES
CC     PLASTIC HINGES AT BOTH ENDS
        IF (ABS(F1-1.0D0).LT.ERROR.AND.ABS(F2-1.0D0).LT.ERROR
     &      .OR.(N.EQ.2)) THEN
          IFLAGH=1
CCC     RESET RCF AND FORM DELMPC
          IF (RCF(IDFRM,1).GT.0.0D0) THEN
            RCF(IDFRM,1)=MMPC(1)
            IF (N.EQ.2) DELMPC(IDFRM,1)=MMPC(1)-JMPC(IA)
          ELSE IF (RCF(IDFRM,1).LT.0.0D0) THEN
            RCF(IDFRM,1)=-MMPC(1)
            IF (N.EQ.2) DELMPC(IDFRM,1)=-MMPC(1)+JMPC(IA)
          ENDIF
          IF (IULTMO.EQ.1.AND.IEND.EQ.1) DELMPC(IDFRM,1)=0.0D0
          IF (RCF(IDFRM,2).GT.0.0D0) THEN
            RCF(IDFRM,2)=MMPC(2)
            IF (N.EQ.2) DELMPC(IDFRM,2)=MMPC(2)-JMPC(IB)
          ELSE IF (RCF(IDFRM,2).LT.0.0D0) THEN
            RCF(IDFRM,2)=-MMPC(2)
            IF (N.EQ.2) DELMPC(IDFRM,2)=-MMPC(2)+JMPC(IB)
          ENDIF
          IF (IULTMO.EQ.1.AND.IEND.EQ.2) DELMPC(IDFRM,2)=0.0D0
CCC     UPDATE JMPC TO MMPC
          IF (N.EQ.2) THEN
            JMPC(IA)=MMPC(1)
            JMPC(IB)=MMPC(2)
          ENDIF
          GO TO 500
        ENDIF
CC     PLASTIC HINGE AT A-TH END ONLY
        IF (ABS(F1-1.0D0).LT.ERROR.AND.ABS(F2-1.0D0).GT.ERROR
     &      .OR.(N.EQ.1.AND.NFLAGH.EQ.1)) THEN
          IFLAGH=2
CCC     RESET RCF AND FORM DELMPC
          IF (RCF(IDFRM,1).GT.0.0D0) THEN
            RCF(IDFRM,1)=MMPC(1)
            IF (N.EQ.1.AND.NFLAGH.EQ.1) DELMPC(IDFRM,1)=MMPC(1)-JMPC(IA)
          ELSE IF (RCF(IDFRM,1).LT.0.0D0) THEN
            RCF(IDFRM,1)=-MMPC(1)
            IF (N.EQ.1.AND.NFLAGH.EQ.1)
     &      DELMPC(IDFRM,1)=-MMPC(1)+JMPC(IA)
          ENDIF
        IF (IULTMO.EQ.1.AND.IEND.EQ.1) DELMPC(IDFRM,1)=0.0D0
CCC     RESET JMPC
        IF (N.EQ.1.AND.NFLAGH.EQ.1) JMPC(IA)=MMPC(1)
        IF (N.EQ.1.AND.NFLAGH.EQ.1) THEN
        DCFI(2)=DCF(2)-DCFTMP(2)
C
        IF (ISOLVE.EQ.1) THEN
        CALL ETSTIF(RCFTMP(IDFRM,3),EET,E,PY)
        CALL SFUNC (RCFTMP(IDFRM,3),S1,S2,LO,EET,INERT)
        XM2TMP=RCFTMP(IDFRM,2)
        PP=ABS(RCFTMP(IDFRM,3))/PY
        CALL ESTIFF (PP,XM2TMP,MP,PHIB)
        RCFI(2)=(EET*INERT/LO)*(S1-S2**2/S1)*DCFI(2)*PHIB
     &          +PHIB*S2*DELMPC(IDFRM,1)/(S1-(1.0D0-PHIB)*S2**2/S1)
        ELSE IF (ISOLVE.EQ.0) THEN
         CALL SFUNC (RCFTMP(IDFRM,3),S1,S2,LO,E,INERT)
         RCFI(2)=(E*INERT/LO)*(S1-S2**2/S1)*DCFI(2)+
     &           S2*DELMPC(IDFRM,1)/S1
        ENDIF

        RCF(IDFRM,2)=RCFTMP(IDFRM,2)+RCFI(2)
        ENDIF
C
          GO TO 500
        ENDIF
CC     PLASTIC HINGE AT B-TH END ONLY
        IF (ABS(F1-1.0D0).GT.ERROR.AND.ABS(F2-1.0D0).LT.ERROR
     &      .OR.(N.EQ.1.AND.NFLAGH.EQ.2)) THEN
          IFLAGH=3
CCC     RESET RCF AND FORM DELMPC
          IF (RCF(IDFRM,2).GT.0.0D0) THEN
            RCF(IDFRM,2)=MMPC(2)
            IF (N.EQ.1.AND.NFLAGH.EQ.2) DELMPC(IDFRM,2)=MMPC(2)-JMPC(IB)
          ELSE IF (RCF(IDFRM,2).LT.0.0D0) THEN
            RCF(IDFRM,2)=-MMPC(2)
            IF (N.EQ.1.AND.NFLAGH.EQ.2)
     &      DELMPC(IDFRM,2)=-MMPC(2)+JMPC(IB)
          ENDIF
        IF (IULTMO.EQ.1.AND.IEND.EQ.2) DELMPC(IDFRM,2)=0.0D0
CCC     RESET JMPC
        IF (N.EQ.1.AND.NFLAGH.EQ.2) JMPC(IB)=MMPC(2)
C
        IF (N.EQ.1.AND.NFLAGH.EQ.2) THEN
        DCFI(1)=DCF(1)-DCFTMP(1)

        IF (ISOLVE.EQ.1) THEN
        CALL ETSTIF(RCFTMP(IDFRM,3),EET,E,PY)
        CALL SFUNC (RCFTMP(IDFRM,3),S1,S2,LO,EET,INERT)
        XM1TMP=RCFTMP(IDFRM,1)
        PP=ABS(RCFTMP(IDFRM,3))/PY
        CALL ESTIFF (PP,XM1TMP,MP,PHIA)
        RCFI(1)=(EET*INERT/LO)*(S1-S2**2/S1)*DCFI(1)*PHIA
     &          +PHIA*S2*DELMPC(IDFRM,2)/(S1-(1.0D0-PHIA)*S2**2/S1)
        ELSE IF (ISOLVE.EQ.0) THEN
        CALL SFUNC (RCFTMP(IDFRM,3),S1,S2,LO,E,INERT)
        RCFI(1)=(E*INERT/LO)*(S1-S2**2/S1)*DCFI(1)+
     &           S2*DELMPC(IDFRM,2)/S1

        ENDIF

        RCF(IDFRM,1)=RCFTMP(IDFRM,1)+RCFI(1)
        ENDIF
C
          GO TO 500
        ENDIF
CC     NO PLASTIC HINGE AT EITHER END
        IF (ABS(F1-1.0D0).GT.ERROR.AND.ABS(F2-1.0D0).GT.ERROR
     &      .OR.(N.EQ.0)) THEN
          IFLAGH=4
          GO TO 500
        ENDIF
500     GO TO (501,502,503,504) IFLAGH
501     CONTINUE
C     PLASTIC HINGES AT BOTH ENDS
CC     UPDATA JHINGE, JMEM AND JEND
        DO 101 JJ=1,JJH
CCC     PLASTIC HINGE FORM AT A-TH END
        IF (IDFRM.EQ.JMEM(JJ).AND.JEND(JJ).EQ.1) GO TO 103
        IF (NCODE(3).EQ.0) GO TO 101
        IF (NCODE(3).EQ.JHINGE(JJ).AND.ISAME(NCODE(3)).EQ.2) GO TO 103
101     CONTINUE
        JJH=JJH+1
        JHINGE(JJH)=NCODE(3)
        JMEM(JJH)=IDFRM
        JEND(JJH)=1
        JMPC(JJH)=MMPC(1)
103     CONTINUE
        DO 104 JJ=1,JJH
CCC     PLASTIC HINGE FORM AT B-TH END
        IF (IDFRM.EQ.JMEM(JJ).AND.JEND(JJ).EQ.2) GO TO 106
        IF (NCODE(6).EQ.0) GO TO 104
        IF (NCODE(6).EQ.JHINGE(JJ).AND.ISAME(NCODE(6)).EQ.2) GO TO 106
104     CONTINUE
        JJH=JJH+1
        JHINGE(JJH)=NCODE(6)
        JMEM(JJH)=IDFRM
        JEND(JJH)=2
        JMPC(JJH)=MMPC(2)
106     CONTINUE
CC     DETERMINE JFLAGH
        NNN=0
        NNFLAG=0
        DO 107 JJ=1,JJH
        IF (IDFRM.EQ.JMEM(JJ).AND.JEND(JJ).EQ.1) THEN
          NNN=NNN+1
          NNFLAG=1
        ENDIF
        IF (IDFRM.EQ.JMEM(JJ).AND.JEND(JJ).EQ.2) THEN
          NNN=NNN+1
          NNFLAG=2
        ENDIF
107     CONTINUE
        IF (NNN.EQ.2) JFLAGH(IDFRM)=1
        IF (NNN.EQ.1.AND.NNFLAG.EQ.1) JFLAGH(IDFRM)=2
        IF (NNN.EQ.1.AND.NNFLAG.EQ.2) JFLAGH(IDFRM)=3
        IF (NNN.EQ.0) JFLAGH(IDFRM)=4
        GO TO 707
502     CONTINUE
C     PLASTIC HINGE AT A-TH END ONLY
CC     UPDATE JHINGE, JMEM AND JEND
        DO 201 JJ=1,JJH
        IF (IDFRM.EQ.JMEM(JJ).AND.JEND(JJ).EQ.1) GO TO 203
        IF (NCODE(3).EQ.0) GO TO 201
        IF (NCODE(3).EQ.JHINGE(JJ).AND.ISAME(NCODE(3)).EQ.2) GO TO 203
201     CONTINUE
        JJH=JJH+1
        JHINGE(JJH)=NCODE(3)
        JMEM(JJH)=IDFRM
        JEND(JJH)=1
        JMPC(JJH)=MMPC(1)
203     CONTINUE
CC     DETERMINE JFLAGH
        NNN=0
        NNFLAG=0
        DO 207 JJ=1,JJH
          IF (IDFRM.EQ.JMEM(JJ).AND.JEND(JJ).EQ.1) THEN
            NNN=NNN+1
            NNFLAG=1
          ENDIF
207     CONTINUE
        IF (NNN.EQ.1.AND.NNFLAG.EQ.1) JFLAGH(IDFRM)=2
        IF (NNN.EQ.0) JFLAGH(IDFRM)=4
        GO TO 707
503     CONTINUE
C     PLASTIC HINGE AT B-TH END ONLY
CC    UPDATE JHINGE, JMEM AND JEND
        DO 301 JJ=1,JJH
        IF (IDFRM.EQ.JMEM(JJ).AND.JEND(JJ).EQ.2) GO TO 303
        IF (NCODE(6).EQ.0) GO TO 301
        IF (NCODE(6).EQ.JHINGE(JJ).AND.ISAME(NCODE(6)).EQ.2) GO TO 303
301     CONTINUE
        JJH=JJH+1
        JHINGE(JJH)=NCODE(6)
        JMEM(JJH)=IDFRM
        JEND(JJH)=2
        JMPC(JJH)=MMPC(2)
303     CONTINUE
CC     DETERMINE JFLAGH
        NNN=0
        NNFLAG=0
        DO 307 JJ=1,JJH
        IF (IDFRM.EQ.JMEM(JJ).AND.JEND(JJ).EQ.2) THEN
          NNN=NNN+1
          NNFLAG=2
        ENDIF
307     CONTINUE
        IF (NNN.EQ.1.AND.NNFLAG.EQ.2) JFLAGH(IDFRM)=3
        IF (NNN.EQ.0) JFLAGH(IDFRM)=4
        GO TO 707
504     CONTINUE
C     NO PLASTIC HINGE AT EITHER END
        JFLAGH(IDFRM)=4
C
707     CONTINUE
C
C    FORMAT STATEMENTS
96      FORMAT ( //,4X,'> MOMENT AT THE A-TH END OF FRAME ELEMENT',I3,
     &' IS GREATER THAN  ',/
     &             ,4X,'> THE PLASTIC MOMENT, THEREFORE, THE LOAD INCRE
     &MENT IS SCALED ',/
     &             ,4X,'> DOWN BY A FACTOR OF',D12.5,29X,'<<')
97      FORMAT ( //,4X,'> MOMENT AT THE B-TH END OF FRAME ELEMENT',I3,
     &' IS GREATER THAN',/
     &             ,4X,'> THE PLASTIC MOMENT. THEREFORE, THE LOAD'
     &' INCREMENT IS SCALED',/
     &             ,4X,'> DOWN BY A FACTOR OF',D12.5)
98      FORMAT ( //,4X,'> P/PY FOR FRAME ELEMENT',I3,' IS GREATER THAN
     &UNITY, THEREFORE, ',/
     &             ,4X,'> LOAD INCREMENT IS REDUCED TO 1/10 OF PREVIOUS
     & VALUE')
99      FORMAT ( //,4X,'> P/PY FOR FRAME ELEMENT',I3,' IS GREATER THAN
     &UNITY <<',/,T8,'THE NODAL FORCE VECTOR AT THIS STATE IS :',/,
     &(T10,6D13.5))
901      FORMAT ( /,4X,'> CONNECTION ROTATION IS GREATER THAN 0.005,
     &THE LOAD INCREMENT IS SCALED')
C
1000    IF(IEND.NE.0) THEN
          THE(IDFRM)=THETAR
          MOM(IDFRM)=RCF(IDFRM,IEND)
        ENDIF
        RETURN
        END
C
C
C
        SUBROUTINE SFUNC (RCFA,S1,S2,LO,E,INERT)
C    THIS SUBROUTINE COMPUTES THE STABILITY FUNCTIONS BASED ON THE AXIAL
C    FORCE IN THE FRAME ELEMENT
C      RCFA     AXIAL FORCE OF THE MEMBER
C      S1,S2    STABILITY COEFFICIENTS
C      LO       LENGTH OF THE UNDISPLACED ELEMENT
C      INERT    MOMENT OF INERTIA
C      AREA     CROSS SECTION AREA
C
        IMPLICIT DOUBLE PRECISION (A-H,K,O-Z)
        DOUBLE PRECISION LO,INERT
        PI=3.141592653589793238462643D0
        RHO=(RCFA*LO**2)/(PI**2*E*INERT)
        IF (ABS(RHO).LE.2.0D0) THEN
        HORHO1=((1.0D-02*RHO+5.43D-01)/(4.0D0+RHO)
     &         +(4.0D-03*RHO+2.85D-01)/(8.183D0+RHO))*RHO**2
        HORHO2=((1.0D-02*RHO+5.43D-01)/(4.0D0+RHO)
     &         -(4.0D-03*RHO+2.85D-01)/(8.183D0+RHO))*RHO**2
        S1=(4.0D0+2.0D0*PI**2*RHO/15.0D0-HORHO1)
        S2=(2.0D0-PI**2*RHO/30.0D0+HORHO2)
        ELSE IF (ABS(RHO).GT.2.0D0) THEN
        KL=PI*SQRT(ABS(RHO))
        IF (RHO.LT.0.0D0) THEN
        DENOM=2.0D0-2.0D0*COS(KL)-KL*SIN(KL)
        S1=(KL*SIN(KL)-KL**2*COS(KL))/DENOM
        S2=(KL**2-KL*SIN(KL))/DENOM
        ELSE IF (RHO.GT.0.0D0) THEN
        DENOM=2.0D0-2.0D0*COSH(KL)+KL*SINH(KL)
        S1=(KL**2*COSH(KL)-KL*SINH(KL))/DENOM
        S2=(KL*SINH(KL)-KL**2)/DENOM
        ENDIF
        ENDIF
        RETURN
        END
C
C
C
C
        SUBROUTINE FRAMEE (KFRAME,XO,YO,E,INERT,AREA,DGF,ICNT,RCF,
     &                     IEND,RKT)
C    THIS SUBROUTINE IS CALLED BY SSTAR TO FORMULATE THE FRAME ELEMENT 
C    STIFFNESS MATRIX, KFRAME, WITH (ICNT=1) OR WITHOUT (ICNT=0) 
C    MODIFICATION FOR THE PRESENCE OF CONNECTION.
C
C      KFRAME   FRAME ELEMENT TANGENT STIFFNESS MATRIX
C      XO       HORIZONTAL PROJECTION OF THE UNDISPLACED ELEMENT
C      YO       VERTICAL PROJECTION OF THE UNDISPLACED ELEMENT
C      INERT    MOMENT OF INERTIA
C      AREA     CROSS SECTION AREA
C      DGF      ARRAY OF ELEMENT DISPLACEMENTS IN GLOBAL COORDINATE
C      RCF      ARRAY OF BASIC FORCES
C      ICNT     FLAG FOR THE PRESENCE OF CONNECTION
C      IEND     ELEMENT END IDENTIFIER
C      RKT      INSTANTANEOUS CONNECTION STIFFNESS
C      IULTMO   FLAG FOR THE ATTAINMENT OF CONNECTION ULTIMATE MOMENT
C
        PARAMETER (MAXDOF=300,MAXTOF=300,MAXROF=100)
        IMPLICIT DOUBLE PRECISION (A-H,K,O-Z)
        DOUBLE PRECISION LO,INERT,MP
        DIMENSION KFRAME(6,6),TFRAME(6,6),TCGF(3,6),
     &            DGF(6),RGS(MAXDOF),RGFPSD(6)
        DIMENSION KCC(3,3),KC(MAXDOF,3,3),RCF(MAXDOF,3),RCFPS(3),
     &            RCFPSD(MAXDOF,3),DELMPC(MAXDOF,2),JFLAGH(MAXDOF)
        DIMENSION PHIAA(MAXDOF),PHIBB(MAXDOF),ETFRM(MAXDOF)
        COMMON /FRAMEB/ ZP,FYF,RGS,NDOFS,IDFRM
        COMMON /RGFPSB/ RGFPSD
        COMMON /FRMIN/ DELMPC,JFLAGH
        COMMON /FRMOUT/ RCFPSD,KC
        COMMON /INESTF/ PHIAA,PHIBB,ETFRM
        COMMON /SOLVEB/ ISOLVE,IGEOIM,ILRFD
C    
C       JFLAGH = 4 - NO PLASTIC HINGES
C                3 - PLASTIC HINGE AT B-TH END
C                2 - PLASTIC HINGE AT A-TH END
C                1 - PLASTIC NINGES AT BOTH ENDS
C 
C    INPUT : DELMPC, RCF, JFLAGH
C    OUTPUT: RGFPSD TO MAIN PROGRAM
C            RCFPSD, KC TO FRMBFC
C
C    INITIALIZATION
        CALL INITM (KFRAME,6,6)
        CALL INITM (TFRAME,6,6)
        CALL INITM (TCGF,3,6)
        CALL INITM (KCC,3,3)
        CALL INITRV (RGFPSD,6)
        CALL INITRV (RCFPS,3)
C
        LO=SQRT(XO**2+YO**2)
        XF=XO+DGF(4)-DGF(1)
        YF=YO+DGF(5)-DGF(2)
        THETA=ATAN2(YF,XF)
CC    CALCULATE THE YIELD LOAD
           IF(ILRFD.EQ.1) THEN
          CPHIC=0.85D0
          CPHIB=0.90D0
          ELSE
          CPHIC=1.0D0
          CPHIB=1.0D0
          ENDIF
        PY=AREA*FYF*CPHIC
CC    EVALUATE S1 AND S2
        RCFA=RCF(IDFRM,3)

        IF (ISOLVE.EQ.0) THEN
        CALL SFUNC (RCFA,S1,S2,LO,E,INERT)
        ET=E
        PHIA=1.0D0
        PHIB=1.0D0
        GO TO 21
        END IF

CCC   APPLY AXIAL STIFFNESS REDUCTION
        CALL ETSTIF(RCFA,ET,E,PY)
        CALL SFUNC (RCFA,S1,S2,LO,ET,INERT)
C
C    DETERMINE PLASTIC HINGE REDUCTION FACTORS
C
        PPY=ABS(RCF(IDFRM,3))/PY
        MP=ZP*FYF*cphib
        XM1=RCF(IDFRM,1)
        XM2=RCF(IDFRM,2)
        CALL ESTIFF(PPY,XM1,MP,PHIA)
        CALL ESTIFF(PPY,XM2,MP,PHIB)
C
C    UPDATE REDUCTION FACTOR FOR PRINTING AT MAIN
C
        PHIAA(IDFRM)=PHIA
        PHIBB(IDFRM)=PHIB
        ETFRM(IDFRM)=ET
        IF (PHIA.LT.0.0010D0) PHIA=0.00010D0
        IF (PHIB.LT.0.0010D0) PHIB=0.00010D0

21      IF (RKT.LT.1.0D-05) RKT=1.0D-05
C
C    CALCULATE PLASTIC MOMENT CAPACITY
C   AISC-LRFD CROSS-SECTIONAL STRENGTH FORMULA
C
        IF (PPY.GE.1.0D0) THEN
          CALL INITM (KFRAME,6,6)
          GO TO 606
        ENDIF
C
        KCC(3,3)=ET*AREA/LO

        KC11=PHIA*(S1-(1.0D0-PHIB)*S2**2/S1)
        KC12=PHIA*PHIB*S2
        KC21=KC12
        KC22=PHIB*(S1-(1.0D0-PHIA)*S2**2/S1)
C
CC    NO CONNECTION IS PRESENT AT EITHER END
        IF (ICNT.EQ.0) THEN
        KCC(1,1)=(ET*INERT/LO)*KC11
        KCC(1,2)=(ET*INERT/LO)*KC12
        KCC(1,3)=0.0D0
        KCC(2,1)=KCC(1,2)
        KCC(2,2)=(ET*INERT/LO)*KC22
        KCC(2,3)=0.0D0
        KCC(3,1)=0.0D0
        KCC(3,2)=0.0D0
CC    CONNECTION IS PRESENT AT A-TH END
        ELSE IF (ICNT.EQ.1.AND.IEND.EQ.1) THEN
        FACTOR=(LO*RKT)/(ET*INERT)+KC11
        KCC(1,1)=(ET*INERT/LO)*(KC11-KC11*KC11/FACTOR)
        KCC(1,2)=(ET*INERT/LO)*(KC12-KC11*KC12/FACTOR)
        KCC(1,3)=0.0D0
        KCC(2,1)=KCC(1,2)
        KCC(2,2)=(ET*INERT/LO)*(KC22-KC12*KC12/FACTOR)
        KCC(2,3)=0.0D0
        KCC(3,1)=0.0D0
        KCC(3,2)=0.0D0
CC    CONNECTION IS PRESENT AT B-TH END
        ELSE IF (ICNT.EQ.1.AND.IEND.EQ.2) THEN
        FACTOR=(LO*RKT)/(ET*INERT)+KC22
        KCC(1,1)=(ET*INERT/LO)*(KC11-KC12*KC12/FACTOR)
        KCC(1,2)=(ET*INERT/LO)*(KC12-KC12*KC22/FACTOR)
        KCC(1,3)=0.0D0
        KCC(2,1)=KCC(1,2)
        KCC(2,2)=(ET*INERT/LO)*(KC22-KC22*KC22/FACTOR)
        KCC(2,3)=0.0D0
        KCC(3,1)=0.0D0
        KCC(3,2)=0.0D0
        ENDIF
CC    FORMULATE THE PSEUDO BASIC FORCE VECTOR, RCFPS, AND
CC    MODIFY THE BASIC FORCE-DISPLACEMENT RELATIONSHIP FOR THE PRESENCE
CC    OF PLASTIC HINGES, IF NECESSARY
        GO TO (601,602,603,604),JFLAGH(IDFRM)
601     CONTINUE
CCC    PLASTIC HINGES AT BOTH ENDS
CCCC    FORM RCFPS
        RCFPS(1)=DELMPC(IDFRM,1)
        RCFPS(2)=DELMPC(IDFRM,2)
        RCFPS(3)=0.0D0
CCCC    MODIFY KCC
        KCC(1,1)=0.0D0
        KCC(1,2)=0.0D0
        KCC(1,3)=0.0D0
        KCC(2,1)=0.0D0
        KCC(2,2)=0.0D0
        KCC(2,3)=0.0D0
        KCC(3,1)=0.0D0
        KCC(3,2)=0.0D0
        GO TO 604
602     CONTINUE
CCC    PLASTIC HINGE AT A-TH END ONLY i.e PHIA=0
CCCC    FORM RCFPS
        RCFPS(1)=DELMPC(IDFRM,1)
        RCFPS(2)=KCC(1,2)*DELMPC(IDFRM,1)/KCC(1,1)
        RCFPS(3)=0.0D0
CCCC    MODIFY KCC
        KCC(2,2)=KCC(2,2)-KCC(1,2)**2/KCC(1,1)
        KCC(1,1)=0.0D0
        KCC(1,2)=0.0D0
        KCC(1,3)=0.0D0
        KCC(2,1)=0.0D0
        KCC(2,3)=0.0D0
        KCC(3,1)=0.0D0
        KCC(3,2)=0.0D0
        GO TO 604
603     CONTINUE
CCC    PLASTIC HINGE AT B-TH END ONLY i.e PHIB=0
CCCC    FORM RCFPS
        RCFPS(1)=KCC(1,2)*DELMPC(IDFRM,2)/KCC(2,2) 
        RCFPS(2)=DELMPC(IDFRM,2)
        RCFPS(3)=0.0D0
CCCC    MODIFY KCC
        KCC(1,1)=KCC(1,1)-KCC(1,2)**2/KCC(2,2)
        KCC(1,2)=0.0D0
        KCC(1,3)=0.0D0
        KCC(2,1)=0.0D0
        KCC(2,2)=0.0D0
        KCC(2,3)=0.0D0
        KCC(3,1)=0.0D0
        KCC(3,2)=0.0D0
604     CONTINUE
C     UPDATE RCFPSD AND KC FOR NEXT ITERATION
        DO 1 I = 1,3
          RCFPSD(IDFRM,I)=RCFPS(I)
          DO 2 J = 1,3
            KC(IDFRM,I,J)=KCC(I,J)
2         CONTINUE
1       CONTINUE
C     FORMULATE INCREMENTAL ELEMENT FORCE-DISPLACEMENT RELATIONSHIP
CC    FORM THE TRANSFORMATION MATRIX, TCGF
        CALL FMTCGF (TCGF,XO,YO,DGF)
CC    FORM PSEUDO ELEMENT FORCE VECTOR, RGFPSD
        CALL MUATV (RGFPSD,TCGF,RCFPS,3,6)
CC    FORM KFRAME
        CALL MUATBA (KFRAME,TCGF,KCC,3,6)
CC    ADD THE CONTRIBUTION TO STIFFNESS DUE TO INITIAL STRESS EFFECT
606     CONTINUE
        DO 10 I=1,3
        IF (I.EQ.1) RCFI=RCF(IDFRM,1)
        IF (I.EQ.2) RCFI=RCF(IDFRM,2)
        IF (I.EQ.3) RCFI=RCF(IDFRM,3)
        IF (I.EQ.1.OR.I.EQ.2) THEN
        TFRAME(1,1)=-2.0D0*RCFI*SIN(THETA)*COS(THETA)/LO**2
        TFRAME(1,2)=RCFI*(COS(THETA)**2-SIN(THETA)**2)/LO**2
        TFRAME(1,3)=0.0D0
        TFRAME(1,4)=-TFRAME(1,1)
        TFRAME(1,5)=-TFRAME(1,2)
        TFRAME(1,6)=0.0D0
        TFRAME(2,2)=TFRAME(1,4)
        TFRAME(2,3)=0.0D0
        TFRAME(2,4)=TFRAME(1,5)
        TFRAME(2,5)=TFRAME(1,1)
        TFRAME(2,6)=0.0D0
        TFRAME(3,3)=0.0D0
        TFRAME(3,4)=0.0D0
        TFRAME(3,5)=0.0D0
        TFRAME(3,6)=0.0D0
        TFRAME(4,4)=TFRAME(1,1)
        TFRAME(4,5)=TFRAME(1,2)
        TFRAME(4,6)=0.0D0
        TFRAME(5,5)=TFRAME(1,4)
        TFRAME(5,6)=0.0D0
        TFRAME(6,6)=0.0D0
CCC    FILL IN LOWER TRIANGLE OF TFRAME
        DO 20 JJ=2,6
        DO 20 II=1,JJ-1
20      TFRAME(JJ,II)=TFRAME(II,JJ)
        ELSE IF (I.EQ.3) THEN
        TFRAME(1,1)=RCFI*SIN(THETA)**2/LO
        TFRAME(1,2)=-RCFI*SIN(THETA)*COS(THETA)/LO
        TFRAME(1,3)=0.0D0
        TFRAME(1,4)=-TFRAME(1,1)
        TFRAME(1,5)=-TFRAME(1,2)
        TFRAME(1,6)=0.0D0
        TFRAME(2,2)=RCFI*COS(THETA)**2/LO
        TFRAME(2,3)=0.0D0
        TFRAME(2,4)=-TFRAME(1,2)
        TFRAME(2,5)=-TFRAME(2,2)
        TFRAME(2,6)=0.0D0
        TFRAME(3,3)=0.0D0
        TFRAME(3,4)=0.0D0
        TFRAME(3,5)=0.0D0
        TFRAME(3,6)=0.0D0
        TFRAME(4,4)=TFRAME(1,1)
        TFRAME(4,5)=TFRAME(1,2)
        TFRAME(4,6)=0.0D0
        TFRAME(5,5)=TFRAME(2,2)
        TFRAME(5,6)=0.0D0
        TFRAME(6,6)=0.0D0
CCC    FILL IN LOWER TRIANGLE OF TFRAME
        DO 30 JJ=2,6
        DO 30 II=1,JJ-1
30      TFRAME(JJ,II)=TFRAME(II,JJ)
        ENDIF
CCC    ADD TFRAME TO KFRAME
        DO 40 III=1,6
        DO 40 JJJ=1,6
40      KFRAME(III,JJJ)=KFRAME(III,JJJ)+TFRAME(III,JJJ)
10      CONTINUE
C
C
1000    RETURN
        END
C
C
C
        SUBROUTINE ESTIFF (PPY,M,MP,PHI)
C    THIS SUBROUTINE CALCULATES THE TWO-SURFACE STIFFNESS 
C    REDUCTION FACTOR, PHI
C    TWO DEGRADATION MODEL MAY BE USED IN THE PRESENT FORMULATION:
C    EMODEL=1  BASED ON CROSS-SECTION STRENGTH CRITERION
C          =2  BASED ON MOMENT CRITERION
C
C      PPY      AXIAL FORCE NORMALIZED BY SQUASH LOAD
C      M        MOMENNT
C      MP       FULL PLASTIC MOMENT
C      PHI      END STIFFNESS REDUCTION FACTOR
C
        IMPLICIT DOUBLE PRECISION (A-H,K,O-Z)
        DOUBLE PRECISION M,MP
        COMMON /PHIBLK/ IPHI  
***********************************
*   SPECIFY THE TYPE OF MODEL
*   IPHI = 1 - INITIAL YIELD SURFACE IS HALF THE LIMITING SURFACE
*              WITH PARABOLIC REDUCTION
*   IPHI = 2 - INITIAL YIELD SURFACE IS A STRAIGHT LINE 
*              FROM P/PY=1.0 to M/MP = 0.5
*   IPHI = 3 - INITIAL YIELD SURFACE IS 0.7 THE LIMITING SURFACE
*              WITH PARABOLIC REDUCTION
*   IPHI = 4 - INITIAL YIELD SURFACE IS 0.7 THE LIMITING SURFACE
*              WITH LINEAR STIFFNESS REDUCTION
*
*   IPHI = 1 IS RECOMMENDED IN THE CONTEXT OF AISC LRFD.
*   SEE DISCUSSION IN THE FOLLOWING ARTICLE:
*   Liew et al. (1992) "Second-Order Refined Plastic Hinge Analysis
*   of Frames,"
************************************
        MODEL=IPHI
************************************
        IF (MODEL.EQ.1) THEN

          EQN=2.0D0*ABS(M)/(9.0D0*MP)
          IF(PPY.LE.EQN) THEN
            ALPHA=PPY/2.0D0+ABS(M)/MP
          ELSE IF (PPY.GT.EQN) THEN
            ALPHA=PPY+8.0D0*ABS(M)/(9.0D0*MP)
          ENDIF
          IF(ALPHA.LE.0.50D0) THEN
            PHI=1.0D0
          ELSE IF(ALPHA.GT.0.5D0.AND.ALPHA.LT.1.0D0) THEN
            PHI=4.0D0*ALPHA*(1.0D0-ALPHA)
          ELSE IF(ALPHA.GE.1.0D0) THEN
            PHI=0.0D0
          ENDIF

        ELSE IF (MODEL.EQ.2) THEN

          IF(PPY.LE.0.20D0) THEN
            ALPHA=2.0D0*ABS(M)/MP+PPY-1.0D0
          ELSE IF (PPY.GT.0.20D0) THEN
            ALPHA=((2.0D0*ABS(M)/MP)/(1.0D0-PPY)-1.0D0)*4.0D0/5.0D0
          ENDIF
          IF(ALPHA.LE.0.0D0) THEN
            PHI=1.0D0
          ELSE IF(ALPHA.GT.0.0D0.AND.ALPHA.LT.1.0D0) THEN
            PHI=1.0D0-ALPHA**2
          ELSE IF(ALPHA.GE.1.0D0) THEN
            PHI=0.0D0
          ENDIF

        ELSE IF (MODEL.EQ.3) THEN

          EQN=2.0D0*ABS(M)/(9.0D0*MP)
          IF(PPY.LE.EQN) THEN
            ALPHA=PPY/2.0D0+ABS(M)/MP
          ELSE IF (PPY.GT.EQN) THEN 
            ALPHA=PPY+8.0D0*ABS(M)/(9.0D0*MP)
          ENDIF
          IF(ALPHA.LE.0.70D0) THEN
            PHI=1.0D0
          ELSE IF(ALPHA.GT.0.7D0.AND.ALPHA.LT.1.0D0) THEN
            PHI=(-(ALPHA**2)+1.4D0*ALPHA-0.4D0)/0.09D0
          ELSE IF(ALPHA.GE.1.0D0) THEN
            PHI=0.0D0
          ENDIF

        ELSE IF (MODEL.EQ.4) THEN

          EQN=2.0D0*ABS(M)/(9.0D0*MP)
          IF(PPY.LE.EQN) THEN
            ALPHA=PPY/2.0D0+ABS(M)/MP
          ELSE IF (PPY.GT.EQN) THEN 
            ALPHA=PPY+8.0D0*ABS(M)/(9.0D0*MP)
          ENDIF
          IF(ALPHA.LE.0.70D0) THEN
            PHI=1.0D0
          ELSE IF(ALPHA.GT.0.7D0.AND.ALPHA.LT.1.0D0) THEN
            PHI=(1.0D0-ALPHA)/0.30D0  
          ELSE IF(ALPHA.GE.1.0D0) THEN
            PHI=0.0D0
          ENDIF
 
        ENDIF
        RETURN
        END

C
C
C
        SUBROUTINE FMTCGF (TCGF,XO,YO,DGF)
C    THIS SUBROUTINE FORMS THE TRANSFORMATION MATRIX, TCGF, FROM THE
C    COROTATIONAL TO GLOBAL COORDINATE FOR THE FRAME ELEMENT
C      TCGF     COORDINATE TRANSFORMATION MATRIX
C      XO       HORIZONTAL PROJECTION OF THE UNDISPLACED ELEMENT
C      YO       VERTICAL PROJECTION OF THE UNDISPLACED ELEMENT
C      DGF      ARRAY OF ELEMENT DISPLACEMENTS IN GLOBAL COORDINATE
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DOUBLE PRECISION LO
        DIMENSION TCGF(3,6),DGF(6)
C    INITIALIZATION
        CALL INITM (TCGF,3,6)
C    FORM TCGF
        XF=XO+DGF(4)-DGF(1)
        YF=YO+DGF(5)-DGF(2)
        THETA=ATAN2(YF,XF)
        LO=SQRT(XO**2+YO**2)
C
        TCGF(1,1)=-SIN(THETA)/LO
        TCGF(1,2)=COS(THETA)/LO
        TCGF(1,3)=1.0D0
        TCGF(1,4)=SIN(THETA)/LO
        TCGF(1,5)=-COS(THETA)/LO
        TCGF(1,6)=0.0D0
        TCGF(2,1)=-SIN(THETA)/LO
        TCGF(2,2)=COS(THETA)/LO
        TCGF(2,3)=0.0D0
        TCGF(2,4)=SIN(THETA)/LO
        TCGF(2,5)=-COS(THETA)/LO
        TCGF(2,6)=1.0D0
        TCGF(3,1)=-COS(THETA)
        TCGF(3,2)=-SIN(THETA)
        TCGF(3,3)=0.0D0
        TCGF(3,4)=COS(THETA)
        TCGF(3,5)=SIN(THETA)
        TCGF(3,6)=0.0D0
C
        RETURN
        END
C
C
C
        SUBROUTINE TRSBFC (RCT,DGT,DGTTMP,XO,YO,E,INERT,AREA,
     &                     RCTTMP)
C    THIS SUBROUTINE FINDS THE BASIC FORCE OF THE TRUSS ELEMENT, RCT,
C    FROM THE TRUSS GLOBAL DISPLACEMENT VECTOR, DGT
C      RCT      ELEMENT BASIC FORCE
C      DGT      ARRAY OF ELEMENT DISPLACEMENTS IN GLOBAL COORDINATE
C      XO       HORIZONTAL PROJECTION OF THE UNDISPLACED ELEMENT
C      YO       VERTICAL PROJECTION OF THE UNDISPLACED ELEMENT
C      E        ELASTIC MODULUS
C      INERT    MOMENT OF INERTIA
C      AREA     CROSS SECTION AREA
C
        PARAMETER (MAXTRS=150)
        IMPLICIT DOUBLE PRECISION (A-H,K,O-Z)
        DOUBLE PRECISION LO,INERT,LAMBDA
        DIMENSION DGT(4),JFAIL(MAXTRS),JJFAIL(MAXTRS)
        DIMENSION RCT(MAXTRS),DGTI(4),TCGT(1,4),
     &            RCTTMP(MAXTRS),DGTTMP(4)
        COMMON /TRUSSB/ FYT,IDTRS,JFAIL,JJFAIL
        COMMON /SOLVEB/ ISOLVE,IGEOIM,ILRFD
C    INITIALIZATION
        DCTI=0.0D0
        LO=SQRT(XO**2+YO**2)
C    EVALUATE THE TRANSFORMATION MATRIX, TCGT
        CALL FMTCGT (TCGT,XO,YO,DGTTMP)
CC   EVALUATE INCREMENTAL GLOBAL DISPLACEMENT VECTOR, DGTI
        DO 1, I=1,4
1       DGTI(I)=DGT(I)-DGTTMP(I)
CC   EVALUATE BASIC INCREMENTAL DISPLACEMENT, DCTI
        DO 2, I=1,4
2       DCTI=DCTI+TCGT(1,I)*DGTI(I)
          IF(ILRFD.EQ.1) THEN
          CPHIC=0.85D0
          CPHIB=0.90D0
          ELSE
          CPHIC=1.0D0
          CPHIB=1.0D0
          ENDIF
        PY=AREA*FYT*CPHIC
        
        IF (ISOLVE.EQ.0) THEN
        ET=E
        ELSE IF (ISOLVE.EQ.1) THEN 
CC   EVALUATE TANGENT STIFFNESS MATRIX BASED ON PREVIOUS LOAD
        CALL ETSTIF(RCTTMP(IDTRS),ET,E,PY)
        ENDIF
CC   EVALUATE BASIC INCREMENTAL FORCE VECTOR, RCTI,
CC   AND BASIC FORCE VECTOR, RCT
        RCTI=ET*AREA*DCTI/LO
        RCT(IDTRS)=RCTTMP(IDTRS)+RCTI
C    CHECK FOR LIMIT STATE
CC    CALCULATE THE YIELD LOAD, PY, AND THE TANGENT MODULUS LOAD, PT
        PI=3.141592653589793238462643D0
        LAMBDA=SQRT((FYT*AREA)/(E*INERT))*LO/PI
        IF (LAMBDA.LE.1.5D0) THEN
        PT=DEXP(-0.419D0*LAMBDA**2)*PY
        ELSE IF (LAMBDA.GT.1.5D0) THEN
        PT=0.877D0*PY/LAMBDA**2
        ENDIF
CC    RE-EVALULATE RCT IF A LIMIT STATE IS REACHED OR EXCEEDED
        IF (JJFAIL(IDTRS).EQ.0) THEN
CCC   AXIAL LOAD IS IN TENSION
        IF (RCT(IDTRS).GE.0.0D0) THEN
        IF (RCT(IDTRS).LT.PY) THEN
        GO TO 1000
        ELSE IF (RCT(IDTRS).GE.PY) THEN
        RCT(IDTRS)=PY
        JFAIL(IDTRS)=2
        ENDIF
CCC   AXIAL LOAD IS IN COMPRESSION
        ELSE IF (RCT(IDTRS).LT.0.0D0) THEN
        IF (ABS(RCT(IDTRS)).LT.PT) THEN
        GO TO 1000
        ELSE IF (ABS(RCT(IDTRS)).GE.PT.AND.ABS(RCT(IDTRS)).LT.PY)
     &  THEN
        RCT(IDTRS)=-PT
        JFAIL(IDTRS)=1
        ELSE IF (ABS(RCT(IDTRS)).GE.PT.AND.ABS(RCT(IDTRS)).GE.PY)
     &  THEN
        RCT(IDTRS)=-PY
        JFAIL(IDTRS)=2
        ENDIF
        ENDIF
        ENDIF
C
        IF (JJFAIL(IDTRS).EQ.1) THEN
        IF (RCT(IDTRS).GE.0.0D0) THEN
        RCT(IDTRS)=PT
        ELSE IF (RCT(IDTRS).LT.0.0D0) THEN
        RCT(IDTRS)=-PT
        ENDIF
        ENDIF
C
        IF (JJFAIL(IDTRS).EQ.2) THEN
        IF (RCT(IDTRS).GE.0.0D0) THEN
        RCT(IDTRS)=PY
        ELSE IF (RCT(IDTRS).LT.0.0D0) THEN
        RCT(IDTRS)=-PY
        ENDIF
        ENDIF
C
1000    RETURN
        END
C
C
C
        SUBROUTINE TRUSSE (KTRUSS,XO,YO,E,AREA,DGT,RCT)
C    THIS SUBROUTINE FORMULATES THE TRUSS STIFFNESS MATRIX, KTRUSS
C      KTRUSS   TRUSS ELEMENT TANGENT STIFFNESS MATRIX
C      XO       HORIZONTAL PROJECTION OF THE UNDISPLACED ELEMENT
C      YO       VERTICAL PROJECTION OF THE UNDISPLACED ELEMENT
C      E        ELASTIC MODULUS
C      AREA     CROSS SECTION AREA
C      DGT      ARRAY OF ELEMENT DISPLACEMENTS IN GLOBAL COORDINATE
C      RCT      ELEMENT BASIC FORCE
C
        PARAMETER (MAXTRS=150)
        IMPLICIT DOUBLE PRECISION (A-H,K,O-Z)
        DOUBLE PRECISION LO
        DIMENSION KTRUSS(4,4),TTRUSS(4,4),TCGT(1,4),DGT(4),
     &            JFAIL(MAXTRS),JJFAIL(MAXTRS)
        DIMENSION KCT(1,1),RCT(MAXTRS),ETT(MAXTRS)
        COMMON /TRUSSB/ FYT,IDTRS,JFAIL,JJFAIL
        COMMON /INETRS/ ETT
        COMMON /SOLVEB/ ISOLVE,IGEOIM,ILRFD
C
C    INPUT:  RCT, DGT
C    OUTPUT: KTRUSS
C
C    INITIALIZATION
        CALL INITM (KTRUSS,4,4)
        CALL INITM (TTRUSS,4,4)
        CALL INITM (TCGT,1,4)
        CALL INITM (KCT,1,1)
        IF (JFAIL(IDTRS).NE.0) GO TO 1000
C    EVALUATE DCT
        XF=XO+DGT(3)-DGT(1)
        YF=YO+DGT(4)-DGT(2)
        THETA=ATAN2(YF,XF)
        LO=SQRT(XO**2+YO**2)
          IF(ILRFD.EQ.1) THEN
          CPHIC=0.85D0
          CPHIB=0.90D0
          ELSE
          CPHIC=1.0D0
          CPHIB=1.0D0
          ENDIF
        PY=AREA*FYT*CPHIC

        IF (ISOLVE.EQ.0) THEN
        ET=E
        ELSE IF (ISOLVE.EQ.1) THEN
C    EVALUATE INELASTIC TANGENT STIFFNESS, ET
        CALL ETSTIF(RCT(IDTRS),ET,E,PY)
C    UPDATE ET FOR PRINTING
        ETT(IDTRS)=ET
        ENDIF

C    FORMULATE INCREMENTAL ELEMENT FORCE-DISPLACEMENT RELATIONSHIP
CC    FORM BASIC FORCE-DISPLACEMENT RELATIONSHIP
        KCT(1,1)=ET*AREA/LO
CC    FORM THE TRANSFORMATION MATRIX, TCGT
        CALL FMTCGT (TCGT,XO,YO,DGT)
CC    FORM KTRUSS
        CALL MUATBA (KTRUSS,TCGT,KCT,1,4)
CC    ADD THE CONTRIBUTION TO STIFFNESS DUE TO INITIAL STRESS EFFECT
        TTRUSS(1,1)=RCT(IDTRS)*SIN(THETA)**2/LO
        TTRUSS(1,2)=-RCT(IDTRS)*SIN(THETA)*COS(THETA)/LO
        TTRUSS(1,3)=-TTRUSS(1,1)
        TTRUSS(1,4)=-TTRUSS(1,2)
        TTRUSS(2,2)=RCT(IDTRS)*COS(THETA)**2/LO
        TTRUSS(2,3)=-TTRUSS(1,2)
        TTRUSS(2,4)=-TTRUSS(2,2)
        TTRUSS(3,3)=TTRUSS(1,1)
        TTRUSS(3,4)=TTRUSS(1,2)
        TTRUSS(4,4)=TTRUSS(2,2)
CCC    FILL IN LOWER TRIANGLE OF TTRUSS
        DO 10 J=2,4
        DO 10 I=1,J-1
10      TTRUSS(J,I)=TTRUSS(I,J)
        DO 20 II=1,4
        DO 20 JJ=1,4
20      KTRUSS(II,JJ)=KTRUSS(II,JJ)+TTRUSS(II,JJ)
C
1000    RETURN
        END
C
C
C
        SUBROUTINE ETSTIF (AXIALF,ET,E,PY)
C     THIS SUBROUTINE CALCULATE THE INELASTIC STIFFNESS REDUCTION
C     FACTOR, ET
C     AXIALF  AXIAL FORCE, + for TENSION and - FOR COMPRESSION
C     ET     INELASTIC STIFFNESS REDUCTION FACTOR
C     E      ELASTIC MODULUS
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        COMMON /ETBLK/  IET
C
C     IET = 1 -- ET STIFFNESS REDUCTION BASED ON THE CRC FORMULA
C     IET = 2 -- ET STIFFNESS REDUCTION BASED ON THE LRFD MANNUAL
C
        MODEL = IET
C
        P=ABS(AXIALF)/PY
        IF (MODEL.EQ.2) THEN

C     AXIAL FORCE IS IN COMPRESSION

        IF(AXIALF.LT.0.0D0) THEN
          IF(P.LE.0.390D0) THEN
            ET=E
          ELSE
            ET=E*(-2.7243*P*LOG(P))
          ENDIF

C     AXIAL FORCE IS IN TENSION
        ELSE IF(AXIALF.GE.0.0D0) THEN
          IF(P.LE.0.500D0) THEN
            ET=E
          ELSE
            ET=E*4.0D0*P*(1.0D0-P)
          ENDIF
        ENDIF

        ELSE IF (MODEL.EQ.1) THEN
 
          IF(P.LE.0.500D0) THEN
            ET=E
          ELSE 
            ET=E*4.0D0*P*(1.0D0-P)
          ENDIF

        ENDIF

C

        RETURN
        END
C
C
C
        SUBROUTINE FMTCGT (TCGT,XO,YO,DGT)
C    THIS SUBROUTINE FORMS THE TRANSFORMATION MATRIX, TCGT, FROM THE
C    COROTATIONAL TO GLOBAL COORDINATE FOR THE TRUSS ELEMENT
C      TCGT     COORDINATE TRANSFORMATION MATRIX
C      XO       HORIZONTAL PROJECTION OF THE UNDISPLACED ELEMENT
C      YO       VERTICAL PROJECTION OF THE UNDISPLACED ELEMENT
C      DGT      ARRAY OF ELEMENT DISPLACEMENTS IN GLOBAL COORDINATE
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION TCGT(1,4),DGT(4)
C    INITIALIZATION
        CALL INITM (TCGT,1,4)
C    FORM TCGT
        XF=XO+DGT(3)-DGT(1)
        YF=YO+DGT(4)-DGT(2)
        THETA=ATAN2(YF,XF)
C
        TCGT(1,1)=-COS(THETA)
        TCGT(1,2)=-SIN(THETA)
        TCGT(1,3)=COS(THETA)
        TCGT(1,4)=SIN(THETA)
C
        RETURN
        END
C
C
C
        SUBROUTINE RKTSTF (RKT,IDFRM,L,MOMT)
C    THIS SUBROUTINE FINDS THE TANGENT STIFFNESS, RKT, OF THE CONNECTION
C      RKT      INSTANTANEOUS CONNECTION STIFFNESS
C      IDFRM    FRAME ELEMENT IDENTIFIER
C      L        CONNECTION IDENTIFIER
C      MOMT     CONNECTION MOMENT
C      THETAR   RELATIVE ROTATION OF BEAM TO COLUMN
C
        PARAMETER (MAXCNT=150,MAXROF=100)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DOUBLE PRECISION MOMT,MU,MM,NPOW
        DIMENSION MU(MAXCNT),NPOW(MAXCNT),RKI(MAXCNT),
     &            ILD(MAXROF),ILDTMP(MAXROF)
        COMMON /CNTB/ MU,NPOW,RKI
        COMMON /ILDB/ ILD,ILDTMP
C
C    OBJECTIVE:  GIVEN MOMT, FIND THETAR
C
        MM=ABS(MOMT/MU(L))
C    DETERMINE REGION OF INTEREST
        IF (ABS(MOMT).LT.MU(L)) IFG=1
        IF (ABS(MOMT).GE.MU(L)) IFG=2
C
        GO TO (12,13) IFG
C
C
12      CONTINUE
C    EVALUATE RKT IF LOADING OCCURS
        IF (ILDTMP(IDFRM).EQ.0) THEN
          RKT=RKI(L)/(1.0D0+MM**NPOW(L)/(1.0D0-MM**NPOW(L)))**
     &        (1.0D0+1.0D0/NPOW(L))
          IF (RKT.GT.0.0D0.AND.RKT.LT.1.0D-05) RKT=0.0D0
C    EVALUATE RKT (=RKI) IF UNLOADING OCCURS
        ELSE IF (ILDTMP(IDFRM).EQ.1) THEN
          RKT=RKI(L)
        ENDIF
        IF (RKT.LT.0.0D0) WRITE (5,1000) IDFRM
        GO TO 2000
13      CONTINUE
C    CONNECTION STIFFNESS REMAINS CONSTANT
        RKT=RKI(L)
        GO TO 2000
C
C    FORMAT STATEMENT
1000    FORMAT ( /,1X,'** CAUTION : TANGENT STIFFNESS OF THE CONNECTION
     &WHICH IS **',/
     &            ,1X,'** CONNECTED TO FRAME ELEMENT',I3,' IS NEGATIVE'
     &,14X,'**')
C
2000    RETURN
        END
C
C
C
        SUBROUTINE SOLVDT (X,DET,TRACE,A,B,N,NDIM,NORDER)
C   THIS SUBROUTINE SOLVES FOR THE UNKNOWN VECTOR X IN THE
C   MATRIX EQUATION A(NXN)*X(NX1)=B(NX1)
C   IN ADDITION, THE DETERMINANT AND TRACE OF MATRIX A IS EVALUATED
C      X        UNKNOWN VECTOR TO BE SOLVED
C      DET      MAGNITUDE OF THE DETERMINANT
C      TRACE    TRACE OF THE STIFFNESS MATRIX
C      A        COEFFICIENT MATRIX
C      B        RIGHT-HAND-SIDE VECTOR
C      N        DIMENSION OF MATRIX A
C      NDIM     DIMENSION OF MATRIX A IN CALLING PROGRAM
C      NORDER   INTEGER VECTOR HOLDING ROW ORDER AFTER PIVOTING
C
        DOUBLE PRECISION A,X,B,DET,TRACE
        DIMENSION A(NDIM,NDIM),B(NDIM),X(NDIM),NORDER(NDIM)
        COMMON /STOPB/ ISTOP
        COMMON /POWERB/ MPOWER,NPOWER
C    INITIALIZATION
        CALL INITRV (X,NDIM)
        ISTOP=0
        DET=0.0D0
        TRACE=0.0D0
        MPOWER=0
        NPOWER=0
C    CALCULATE TRACE OF A
        DO 10 I=1,N
        TRACE=TRACE+A(I,I)
10      CONTINUE
        IF (TRACE.GE.1.0D0) THEN
102     TRACE=TRACE/10.0D0
        MPOWER=MPOWER+1
        IF (TRACE.GE.1.0D0) GO TO 102
        ENDIF
C    CALCULATE DETERMINANT OF A
        CALL DETERM (DET,A,N,NDIM,NORDER)
        IF (ISTOP.EQ.1) GO TO 1000
C    SOLVE FOR THE VECTOR X
        CALL SOLVLU (X,A,B,N,NDIM,NORDER)
C
1000    RETURN
        END
C
C
C
        SUBROUTINE DETERM (DET,A,N,NDIM,NORDER)
C    THIS SUBROUTINE FINDS THE DETERMINANT OF MATRIX A(NXN).
C    MATRIX A IS FIRST DECOMPOSED INTO A=LU BY CHOLESKY DECOMPOSITION,
C    WHERE L IS A LOWER TRIANGULAR MATRIX AND U IS AN UPPER TRIANGULAR
C    MATRIX WITH 1'S ON ITS DIAGONAL. THE DETERMINANT OF A IS THEN
C    OBTAINED AS THE PRODUCT OF THE DIAGONAL ELEMENTS OF THE L MATRIX.
C    PARTIAL PIVOTING IS USED IN THE DECOMPOSITION PROCESS
C      A        THE N X N COEFFICIENT MATRIX
C      N        THE NUMBER OF EQUATIONS
C      NDIM     THE DIMENSION OF A IN THE CALLING PROGRAM
C      NORDER   INTEGER VECTOR HOLDING ROW ORDER AFTER PIVOTING
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION A(NDIM,NDIM),NORDER(NDIM)
        COMMON /STOPB/ ISTOP
        COMMON /POWERB/ MPOWER,NPOWER
        ISTOP=0
C    ESTABLISH INITIAL ORDERING IN NORDER
        DO 10 I=1,N
        NORDER(I)=I
10      CONTINUE
        M=0
C    DO PIVOTING FOR FIRST COLUMN
        CALL PVTM (A,N,NDIM,NORDER,1,M)
C    PRINT ERROR MESSAGE IF PIVOT ELEMENT IS TOO SMALL
        IF (ABS(A(1,1)).LT.1.0D-10) GO TO 99
C    COMPUTE ELEMENTS FOR FIRST ROW OF U
        DO 20 KCOL=2,N
        A(1,KCOL)=A(1,KCOL)/A(1,1)
20      CONTINUE
C    COMPLETE THE COMPUTING OF L AND U ELEMENTS.  THE GENERAL
C    PLAN IS TO COMPUTE A COLUMN OF L'S, THEN CALL PIVOT TO
C    INTERCHANGE ROWS, AND THEN GET A ROW OF U'S
        NM1=N-1
        DO 80 JCOL=2,NM1
C    FIRST COMPUTE A COLUMN OF L'S
        JM1=JCOL-1
        DO 50 IROW=JCOL,N
        SUM=0.0D0
        DO 40 KCOL=1,JM1
        SUM=SUM+A(IROW,KCOL)*A(KCOL,JCOL)
40      CONTINUE
        A(IROW,JCOL)=A(IROW,JCOL)-SUM
50      CONTINUE
C    INTERCHANGE ROWS IF NECESSARY
        CALL PVTM (A,N,NDIM,NORDER,JCOL,M)
        IF (ABS(A(JCOL,JCOL)).LT.1.0D-10) GO TO 99
C    NOW GET A ROW OF U'S
        JP1=JCOL+1
        DO 70 KCOL=JP1,N
        SUM=0.0D0
        DO 60 IROW=1,JM1
        SUM=SUM+A(JCOL,IROW)*A(IROW,KCOL)
60      CONTINUE
        A(JCOL,KCOL)=(A(JCOL,KCOL)-SUM)/A(JCOL,JCOL)
70      CONTINUE
80      CONTINUE
C    GET LAST ELEMENT IN THE L MATRIX
        SUM=0.0D0
        DO 90 KCOL=1,NM1
        SUM=SUM+A(N,KCOL)*A(KCOL,N)
90      CONTINUE
        A(N,N)=A(N,N)-SUM
C    COMPUTE DETERMINANT
        DET=1.0D0
        DO 101 IJ=1,N
        DET=DET*A(IJ,IJ)
        IF (ABS(DET).GE.1.0D0) THEN
102     DET=DET/10.0D0
        NPOWER=NPOWER+1
        IF (ABS(DET).GE.1.0D0) GO TO 102
        ENDIF
101     CONTINUE
C    CHANGE SIGN, IF NECESSARY, DUE TO ROW INTERCHANGE
        SIGN=-1.0D0
        DET=SIGN**M*DET
        RETURN
99      WRITE (5,100)
C    FORMAT STATEMENT
100     FORMAT ( //,1X,'>> THE TANGENT STIFFNESS MATRIX IS SINGULAR, THE
     & STRUCTURE MAY BE UNSTABLE <<')
        ISTOP=1
C
        RETURN
        END
C
C
C
        SUBROUTINE PVTM (A,N,NDIM,NORDER,JCOL,M)
C    THIS SUBROUTINE FINDS THE LARGEST ELEMENT FOR PIVOT IN
C    JCOL OF THE MATRIX A, PERFORMS INTERCHANGES OF ELEMENTS
C    IN A AND ALSO INTERCHANGES THE ELEMENTS IN THE VECTOR
C    NORDER
C      A        COEFFICIENT MATRIX WHOSE ROWS ARE TO BE INTERCHANGED
C      N        NUMBER OF EQUATIONS
C      NDIM     THE DIMENSION OF A IN THE CALLING PROGRAM
C      NORDER   INTEGER VECTOR TO HOLD ROW ORDERING
C      JCOL     COLUMN OF A BEING SEARCH FOR PIVOT ELEMENT
C      M        INDEX FOR SIGN CHANGE FOR DETERMINANT
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION A(NDIM,NDIM),NORDER(NDIM)
C    FIND PIVOT ROW, CONSIDERING ONLY THE ELEMENTS ON OR BELOW
C    THE DIAGONAL
        IPVT=JCOL
        BIG=ABS(A(JCOL,JCOL))
        JP1=JCOL+1
        DO 10 IROW=JP1,N
        ANEXT=ABS(A(IROW,JCOL))
        IF (ANEXT.GT.BIG) IPVT=IROW
10      CONTINUE
C    INTERCHANGE ROW ELEMENTS IN THE ROW WHOSE NUMBER EQUALS JCOL
C    WITH THE PIVOT ROW UNLESS PIVOT ROW IS JCOL
        IF (IPVT.EQ.JCOL) GO TO 25
        M=M+1
        DO 20 KCOL=1,N
        SAVE=A(JCOL,KCOL)
        A(JCOL,KCOL)=A(IPVT,KCOL)
        A(IPVT,KCOL)=SAVE
20      CONTINUE
C    SWITCH ELEMENT IN THE VECTOR NORDER
        ISAVE=NORDER(JCOL)
        NORDER(JCOL)=NORDER(IPVT)
        NORDER(IPVT)=ISAVE
C
25      RETURN
        END
C
C
C
        SUBROUTINE SOLVLU (X,LU,B,N,NDIM,NORDER)
C    THIS SUBROUTINE SOLVES FOR THE VECTOR X IN THE MATRIX
C    EQUATION AX=B, AFTER A HAS BEEN DECOMPOSED INTO A LOWER
C    TRIANGULAR MATRIX L AND AN UPPER TRIANGULAR MATRIX U.
C    THE ELEMENTS OF THE B VECTOR WILL BE REARRANGED IF THE
C    ROWS OF A WERE INTERCHANGED.
C      LU       THE LU EQUIVALENCE OF THE COEFFICIENT MATRIX
C      B        INPUT VECTOR
C      X        SOLUTION VECTOR
C      N        NUMBER OF EQUATIONS
C      NDIM     THE DIMENSION OF A IN THE CALLING PROGRAM
C      NORDER   INTEGER ARRAY OF ROW ORDER AS REARRANGED DURING
C               PIVOTING
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION LU(NDIM,NDIM),B(NDIM),X(NDIM),NORDER(NDIM)
        DOUBLE PRECISION LU
C    REARRANGE THE ELEMENTS OF THE B VECTOR AND STORE IT IN X
        DO 10 I=1,N
        J=NORDER(I)
        X(I)=B(J)
10      CONTINUE
C    COMPUTE THE B' VECTOR, STORING BACK IN X
        X(1)=X(1)/LU(1,1)
        DO 50 IROW=2,N
        IM1=IROW-1
        SUM=0.D00
        DO 40 JCOL=1,IM1
        SUM=SUM+LU(IROW,JCOL)*X(JCOL)
40      CONTINUE
        X(IROW)=(X(IROW)-SUM)/LU(IROW,IROW)
50      CONTINUE
C    COMPUTE THE SOLUTION VECTOR
        DO 70 IROW=2,N
        NVBL=N-IROW+1
        SUM=0.D00
        NP1=NVBL+1
        DO 60 JCOL=NP1,N
        SUM=SUM+LU(NVBL,JCOL)*X(JCOL)
60      CONTINUE
        X(NVBL)=X(NVBL)-SUM
70      CONTINUE
C
        RETURN
        END
C
C
C
        SUBROUTINE ASSEMB (KSTRCT,KELEMT,NCODE,NDOFE,NDIM)
C      THIS SUBROUTINE ASSEMBLES THE STRUCTURE STIFFNESS MATRIX, KSTRCT
C      KSTRCT  STRUCTURE TANGENT STIFFNESS MATRIX
C      KELEMT   ELEMENT TANGENT STIFFNESS MATRIX
C      NCODE    ARRAY OF ELEMENT COORDINATE
C      NDOFE    NUMBER OF DEGREES OF FREEDOM OF THE ELEMENT
C      NDIM     DIMENSION OF KSTRCT IN TH CALLING PROGRAM
C
        IMPLICIT DOUBLE PRECISION (A-H,K,O-Z)
        DIMENSION KSTRCT(NDIM,NDIM),KELEMT(NDOFE,NDOFE),NCODE(NDOFE)
C    ASSEMBLAGE
        DO 10 I=1,NDOFE
        IROW=NCODE(I)
        IF (IROW.EQ.0) GO TO 10
        DO 20 J=1,NDOFE
        ICOL=NCODE(J)
        IF (ICOL.EQ.0) GO TO 20
        KSTRCT(IROW,ICOL)=KSTRCT(IROW,ICOL)+KELEMT(I,J)
20      CONTINUE
10      CONTINUE
C
        RETURN
        END
C
C
C
        SUBROUTINE FMRGSP (RGSPSD,RGE,NCODE,NDOFE,NDIM)
C    THIS SUBROUTINE FORMS THE PSEUDO STRUCTURE FORCE VECTOR IN
C    GLOBAL COORDINATE
C      RGSPSD   STRUCTURE PSEUDO FORCE VECTOR
C      RGE      ELEMENT PSEUDO FORCE VECTOR
C      NCODE    ARRAY OF ELEMENT COORDINATE
C      NDOFE    NUMBER OF DEGREES OF FREEDOM OF THE ELEMENT
C      NDIM     DIMENSION OF RGSPSD IN THE CALLING PROGRAM
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION RGSPSD(NDIM),RGE(NDOFE),NCODE(NDOFE)
C    FORM THE PSEUDO ELEMENT FORCE VECTOR
        DO 10 I=1,NDOFE
        II=NCODE(I)
        IF (II.EQ.0) GO TO 10
        RGSPSD(II)=RGSPSD(II)+RGE(I)
10      CONTINUE
C
        RETURN
        END
C
C
C
        SUBROUTINE INITM (A,MA,NA)
C    THIS SUBROUTINE INITIALIZES A MATRIX OF SIZE MAXNA
        DOUBLE PRECISION A
        DIMENSION A(MA,NA)
        DO 10 I=1,MA
        DO 10 J=1,NA
10      A(I,J)=0.D00
        RETURN
        END
C
C
C
        SUBROUTINE INITRV (RV,MV)
C    THIS SUBROUTINE INITIALIZES A REAL VECTOR OF SIZE MVX1
        DOUBLE PRECISION RV
        DIMENSION RV(MV)
        DO 10 I=1,MV
10      RV(I)=0.D00
        RETURN
        END
C
C
C
        SUBROUTINE INITIV (IV,MV)
C    THIS SUBROUTINE INITIALIZES AN INTEGER VECTOR OF SIZE MVX1
        DIMENSION IV(MV)
        DO 10 I=1,MV
10      IV(I)=0
        RETURN
        END
C
C
C
        SUBROUTINE SUMDIF (ANS,E,F,M,N,MDIM,NDIM,K)
C    THIS SUBROUTINE FINDS THE SUM (K=1) OR DIFFERENCE (K=-1)
C    OF TWO MATRICES E(MXN) AND F(MXN)
        IMPLICIT DOUBLE PRECISION (A-H,K,O-Z)
        DIMENSION E(MDIM,NDIM),F(MDIM,NDIM),ANS(MDIM,NDIM)
        DO 10 IROW=1,M
        DO 10 ICOL=1,N
10      ANS(IROW,ICOL)=E(IROW,ICOL)+K*F(IROW,ICOL)
        RETURN
        END
C
C
C
        SUBROUTINE MUATBA (ATBA,A,B,M,N)
C    THIS SUBROUTINE FINDS THE MATRIX PRODUCT
C    A TRANSPOSE(NXM)*B(MXM)*A(MXN)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION A(M,N),B(M,M),ATBA(N,N)
        DO 10 IROW=1,N
        DO 10 ICOL=1,N
        ATBA(IROW,ICOL)=0.0D0
        DO 20 K=1,M
        DO 20 L=1,M
20      ATBA(IROW,ICOL)=ATBA(IROW,ICOL)+A(K,IROW)*B(K,L)*A(L,ICOL)
10      CONTINUE
        RETURN
        END
C
C
C
        SUBROUTINE MUATV (ATV,A,V,M,N)
C    THIS SUBROUTINE FINDS THE MATRIX PRODUCT
C    A TRANSPOSE(NXM)*V(M)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION A(M,N),V(M),ATV(N)
        DO 10 IROW=1,N
        ATV(IROW)=0.0D0
        DO 20 L=1,M
20      ATV(IROW)=ATV(IROW)+A(L,IROW)*V(L)
10      CONTINUE
        RETURN
        END
C
C
C
        SUBROUTINE MUATBC (ATBC,A,B,C,M,N,MM,NN)
C    THIS SUBROUTINE FINDS THE MATRIX PRODUCT
C    A TRANSPOSE(NXM)*B(MXMM)*C(MMXNN)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION A(M,N),B(M,MM),C(MM,NN),ATBC(N,NN)
        DO 10 IROW=1,N
        DO 10 ICOL=1,NN
        ATBC(IROW,ICOL)=0.0D0
        DO 20 K=1,M
        DO 20 L=1,MM
20      ATBC(IROW,ICOL)=ATBC(IROW,ICOL)+A(K,IROW)*B(K,L)*C(L,ICOL)
10      CONTINUE
        RETURN
        END
C
C
C
        SUBROUTINE MULTMV (C,A,B,M,N)
C    THIS SUBROUTINE FINDS THE PRODUCT OF
C    MATRIX A(MXN) AND COLUMN VECTOR B(N)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION A(M,N),B(N),C(M)
        DO 10 I=1,M
        C(I)=0.0D0
        DO 10 J=1,N
10      C(I)=C(I)+A(I,J)*B(J)
        RETURN
        END
C
C
C
        SUBROUTINE MULTVM (C,B,A,M,N)
C    THIS SUBROUTINE FINDS THE PRODUCT OF
C    ROW VECTOR B(M) AND MATRIX A(MXN)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION A(M,N),B(M),C(N)
        DO 10 J=1,N
        C(J)=0.0D0
        DO 10 I=1,M
10      C(J)=C(J)+B(I)*A(I,J)
        RETURN
        END
C
C
C
        SUBROUTINE MUCVRV (PROD,C,D,M,N)
C    THIS SUBROUTINE FINDS THE PRODUCT OF A COLUMN VECTOR C(M) AND
C    A ROW VECTOR D(N)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION C(M),D(N),PROD(M,N)
        DO 10 IROW=1,M
        DO 10 ICOL=1,N
10      PROD(IROW,ICOL)=C(IROW)*D(ICOL)
        RETURN
        END
